@@ -163,4 +163,12 @@ bool UI_askAvisynthPort(uint32_t &port)
 {
 	return false;
 }
+bool  	UI_reset(void)
+{
+    return true;
+}
+bool UI_setDisplayName(char const*)
+{
+        return true;
+}
 // EOF
