@@ -185,6 +185,9 @@ public:
             
             bool        copyToClipBoard(uint64_t startTime, uint64_t endTime);
             bool        pasteFromClipBoard(uint64_t currentTime);
+            bool        dumpClipBoard();
+protected:
+            void        dumpSegmentsInternal(ListOfSegments &l);
 };
 
 #endif
