@@ -11,8 +11,8 @@ int     A_delete(uint32_t start, uint32_t end);
 void    A_externalAudioTrack(const char *trackIndex, const char *filename );
 uint8_t A_rebuildKeyFrame (void);
 void    A_openBrokenAvi (const char *name);
-int     A_openAvi2 (const char *name, uint8_t mode);
-int     A_appendAvi (const char *name);
+int     A_openVideo2 (const char *name, uint8_t mode);
+int     A_appendVideo (const char *name);
 void    A_videoCheck( void);
 void	A_setPostproc( void );
 void    A_Resync(void);
@@ -29,12 +29,12 @@ void    A_ResetMarkers(void);
 void    A_Rewind(void);
 void    A_jog(void);
 uint8_t A_jumpToTime(uint32_t hh,uint32_t mm,uint32_t ss,uint32_t ms);
-int 			A_openAvi		(const char *name);
-int             A_saveAudio	(const char *name);
-void 			A_saveAudioDecoded	(const char *name);
-void 			A_saveAVI		(const char *name);
-void 			A_playAvi		(void);
-void            A_queueJob      (void);
+int 	A_openVideo		(const char *name);
+int     A_saveAudio	(const char *name);
+void 	A_saveAudioDecoded	(const char *name);
+void 	A_saveAVI		(const char *name);
+void 	A_playAvi		(void);
+void    A_queueJob      (void);
 int  A_saveAudioCopy (const char *name);
 int  A_saveJpg (const char *name);
 int  A_saveBunchJpg(const char *name);
