@@ -41,9 +41,9 @@
 #ifdef USE_SDL
 extern "C" {
 	#ifdef __HAIKU__
-	#include "SDL/SDL.h"
+	#include "SDL2/SDL.h"
 	#else
-	#include "SDL.h"
+	#include "SDL2/SDL.h"
 	#endif
 }
 
@@ -91,7 +91,7 @@ extern bool UI_End(void);
 extern void cleanUp (void);
 
 extern ADM_UI_TYPE UI_GetCurrentUI(void);
-
+static bool performOnExit=true;
 
 #if !defined(NDEBUG) && defined(FIND_LEAKS)
 extern const char* new_progname;
@@ -100,6 +100,12 @@ extern const char* new_progname;
 void onexit(void);
 int startAvidemux(int argc, char *argv[]);
 bool isPortableMode(int argc, char *argv[]);
+#ifdef main
+extern "C"
+{
+int main(int _argc, char *_argv[]);
+}
+#endif // main
 
 int main(int _argc, char *_argv[])
 {
@@ -126,6 +132,7 @@ int main(int _argc, char *_argv[])
 	new_progname = argv[0];
 #endif
 
+
 	int exitVal = startAvidemux(argc, argv);
 
 #ifdef _WIN32
@@ -165,7 +172,7 @@ static const char *getUISpecifSubfolder()
 int startAvidemux(int argc, char *argv[])
 {
     printf("*************************\n");
-    printf("  Avidemux v"ADM_VERSION);
+    printf("  Avidemux v%s", ADM_VERSION);
 
 #if defined( ADM_SUBVERSION )
 #define MKSTRING(x) x
@@ -234,22 +241,13 @@ int startAvidemux(int argc, char *argv[])
         setPrefsDefault();
     }
     CpuCaps::init();
-
-#ifdef USE_SDL
-	uint32_t videoDevice = RENDER_LAST;
-
-	prefs->get(VIDEODEVICE, &videoDevice);
-
-	initSdl(videoDevice);
-#endif
-
-	atexit(onexit);
+    atexit(onexit);
 
 #ifdef _WIN32
     win32_netInit();
 #endif
 
-	video_body = new ADM_Composer;
+    video_body = new ADM_Composer;
 
     UI_Init(argc, argv);
     AUDMEncoder_initDither();
@@ -319,7 +317,25 @@ int startAvidemux(int argc, char *argv[])
     printf("Cannot use LIBVA in cli mode %d,%d\n",ADM_UI_TYPE_BUILD,ADM_UI_CLI);
   #endif
 #endif    
-        
+
+#ifdef USE_SDL
+    char *drv=NULL;
+    printf("Probing for SDL...\n");
+    std::string sdlDriver=std::string("dummy");
+    if(prefs->get(FEATURES_SDLDRIVER,&drv))
+    {
+        if(drv)
+        {
+            if(strlen(drv))
+            {
+                sdlDriver=std::string(drv);
+            }
+            ADM_dezalloc(drv);
+        }
+    }
+    printf("Calling initSDL with driver=%s\n",sdlDriver.c_str());
+    initSdl(sdlDriver);
+#endif        
     //
     
     ADM_mx_loadPlugins(mxPlugins);
@@ -371,13 +387,18 @@ int startAvidemux(int argc, char *argv[])
     printf("Normal exit\n");
     return 0;
 }
-
+void disableExitHandler()
+{
+    performOnExit=false;
+}
 /**
  * 
  */
 void onexit( void )
 {
     printf("Cleaning up\n");
+    if(!performOnExit)
+        return;
     if(video_body) 
         video_body->cleanup ();
     delete video_body;
@@ -463,5 +484,4 @@ bool isPortableMode(int argc, char *argv[])
 
 	return portableMode;
 }
-
 //EOF
