@@ -51,7 +51,7 @@ using std::string;
 #define NB_TREE 8
 #define myFg 0xFF
 #define myBg 0xF0
-static int max=0;
+
 /******************************************************/
 #define ALL_FILTER_BASE       1000
 #define ACTIVE_FILTER_BASE    3000
@@ -59,73 +59,95 @@ static int max=0;
 #define nb_active_filter ADM_vf_getSize()
 
 /*******************************************************/
+#define zprintf(...) {}
+//#define NO_DELEGATE
 
 extern ADM_Composer *video_body;
 
 FilterItemEventFilter::FilterItemEventFilter(QWidget *parent) : QObject(parent) {}
-
+/**
+ * 
+ * @param object
+ * @param event
+ * @return 
+ */
 bool FilterItemEventFilter::eventFilter(QObject *object, QEvent *event)
 {
-	QAbstractItemView *view = qobject_cast<QAbstractItemView*>(parent());
-
-	if (event->type() == QEvent::KeyPress || event->type() == QEvent::MouseButtonPress)
-	{
-		QCoreApplication::sendEvent(view, event);
-
-		return true;
-	}
-	else if (event->type() == QEvent::FocusIn)
-	{
-		view->setFocus();
-
-		return true;
-	}
-	else
-		return QObject::eventFilter(object, event);
+    zprintf("Object : %p\n",object);
+    zprintf("Parent : %p\n",parent());
+#if !defined(NO_EVENT_FILTER)
+    QListWidget *list=qobject_cast<QListWidget*>(parent());
+    QAbstractItemView *view = qobject_cast<QAbstractItemView*>(parent());
+    //printf("Event %d\n",event->type());
+    switch(event->type())
+    {
+        case QEvent::KeyPress :
+        case QEvent::MouseButtonPress:
+        //case QEvent::ContextMenu:
+            QCoreApplication::sendEvent(list, event);
+            return true;
+        case QEvent::FocusIn:       
+            list->setFocus();
+            return true;
+        default:
+            return QObject::eventFilter(object, event);
+    }
+    return false;
+#else
+      return QObject::eventFilter(object, event);
+#endif
 }
-
+/**
+ * 
+ * @param parent
+ */
 FilterItemDelegate::FilterItemDelegate(QWidget *parent) : QItemDelegate(parent)
 {
-	filter = new FilterItemEventFilter(parent);
+    zprintf("Delegate : %p\n",this);
+    filter = new FilterItemEventFilter(parent);
 }
-
+/**
+ * 
+ * @param painter
+ * @param option
+ * @param index
+ */
 void FilterItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
-	QAbstractItemView *view = qobject_cast<QAbstractItemView*>(parent());
-	QLabel *label;
+    QAbstractItemView *view = qobject_cast<QAbstractItemView*>(parent());
+    QLabel *label;
 
-	if (view->indexWidget(index) == 0)
-	{
-		label = new QLabel();
+    if (view->indexWidget(index) == 0)
+    {
+            label = new QLabel();
+            label->installEventFilter(filter);
+            label->setAutoFillBackground(true);
+            label->setFocusPolicy(Qt::TabFocus);
+            label->setText(index.data().toString());
+            view->setIndexWidget(index, label);
+    }
 
-		label->installEventFilter(filter);
-		label->setAutoFillBackground(true);
-		label->setFocusPolicy(Qt::TabFocus);
-		label->setText(index.data().toString());
-		view->setIndexWidget(index, label);
-	}
+    label = (QLabel*)view->indexWidget(index);
 
-	label = (QLabel*)view->indexWidget(index);
-
-	if (option.state & QStyle::State_Selected)
-		if (option.state & QStyle::State_HasFocus)
-			label->setBackgroundRole(QPalette::Highlight);
-		else
-			label->setBackgroundRole(QPalette::Window);
-	else
-		label->setBackgroundRole(QPalette::Base);
+    if (option.state & QStyle::State_Selected)
+            if (option.state & QStyle::State_HasFocus)
+                    label->setBackgroundRole(QPalette::Highlight);
+            else
+                    label->setBackgroundRole(QPalette::Window);
+    else
+            label->setBackgroundRole(QPalette::Base);
 }
 /**
     \fn preview
 */
 void filtermainWindow::preview(bool b)
 {
-   QListWidgetItem *item=activeList->currentItem();
-   if(!item)
-   {
-      printf("No selection\n");
-      return;
-   }
+    QListWidgetItem *item=activeList->currentItem();
+    if(!item)
+    {
+       printf("No selection\n");
+       return;
+    }
 
      int itag=item->type();
      ADM_assert(itag>=ACTIVE_FILTER_BASE);
@@ -134,28 +156,25 @@ void filtermainWindow::preview(bool b)
      ADM_info("Rank : %d\n",itag);
      ADM_coreVideoFilter     *filter=ADM_vf_getInstance(itag);
      ADM_assert(filter);
-	if (previewDialog)
-		previewDialog->resetVideoStream(filter);
-	else
-	{
-		previewDialog = new Ui_seekablePreviewWindow(this, filter, 0);
-		connect(previewDialog, SIGNAL(accepted()), this, SLOT(closePreview()));
-#if 0
-		if (previewDialogX != INT_MIN)
-			previewDialog->move(previewDialogX, previewDialogY);
-#endif
-	}
-	previewDialog->show();
+    if (previewDialog)
+            previewDialog->resetVideoStream(filter);
+    else
+    {
+        previewDialog = new Ui_seekablePreviewWindow(this, filter, 0);
+        connect(previewDialog, SIGNAL(accepted()), this, SLOT(closePreview()));
+    }
+    previewDialog->show();
 }
-
+/**
+ * \fn closePreview
+ */
 void filtermainWindow::closePreview()
 {
-	if (previewDialog)
-	{
-		delete previewDialog;
-		previewDialog = NULL;
-	}
-
+    if (previewDialog)
+    {
+        delete previewDialog;
+        previewDialog = NULL;
+    }
 }
 
 /**
@@ -201,9 +220,16 @@ void filtermainWindow::add( bool b)
         {
             ADM_warning("Cannot add filter from tag\n");
         }
-
    }
+}
 
+/**
+    \fn    Add
+    \brief Right click on an available filer
+*/
+void filtermainWindow::removeAction(void)
+{
+   remove(true);
 }
 /**
         \fn     remove( bool b)
@@ -211,119 +237,85 @@ void filtermainWindow::add( bool b)
 */
 void filtermainWindow::remove( bool b)
 {
-   /* Get selection if any */
-  /* Now that we have the tab, get the selection */
+    int itag=getTagForActiveSelection();
+    if(-1==itag)
+        return;
+    ADM_info("Deleting item %d\n",itag);
+    ADM_vf_removeFilterAtIndex(itag);
+    buildActiveFilterList ();
+    if(nb_active_filter)
+    {
+          setSelected(nb_active_filter-1);
+    }
+}
+/**
+    \fn    Add
+    \brief Right click on an available filer
+*/
+void filtermainWindow::configureAction(void)
+{
+    configure(true);
+}
+
+/**
+ * \fn getTagForActiveSelection
+ */
+int filtermainWindow::getTagForActiveSelection()
+{
    QListWidgetItem *item=activeList->currentItem();
    if(!item)
    {
       ADM_warning("No selection\n");
-      return;
+      return -1;
    }
 
-     int itag=item->type();
-     ADM_assert(itag>=ACTIVE_FILTER_BASE);
-     itag-=ACTIVE_FILTER_BASE;
-     ADM_info("Deleting item %d\n",itag);
-     ADM_vf_removeFilterAtIndex(itag);
-     buildActiveFilterList ();
-     if(nb_active_filter)
-     {
-          setSelected(nb_active_filter-1);
-     }
+    int itag=item->type();
+    ADM_assert(itag>=ACTIVE_FILTER_BASE);
+    itag-=ACTIVE_FILTER_BASE;
+    /* Filter 0 is the decoder ...*/ 
+    return itag;
 }
-#if 1
-#define MAKE_BUTTON(button,call) \
-void filtermainWindow::button( bool b)  {}
-#else
-#define MAKE_BUTTON(button,call) \
-void filtermainWindow::button( bool b) \
-{ \
-    call(); \
-    getFirstVideoFilter (); \
-    buildActiveFilterList ();  \
-	setSelected(nb_active_filter - 1); \
-}
-#endif
-MAKE_BUTTON(DVD,setDVD)
-MAKE_BUTTON(VCD,setVCD)
-MAKE_BUTTON(SVCD,setSVCD)
-MAKE_BUTTON(halfD1,setHalfD1)
 /**
         \fn     configure( bool b)
         \brief  Configure the selected active filter
 */
 void filtermainWindow::configure( bool b)
 {
-   /* Get selection if any */
-  /* Now that we have the tab, get the selection */
-   QListWidgetItem *item=activeList->currentItem();
-   if(!item)
-   {
-      ADM_warning("No selection\n");
-      return;
-   }
-
-     int itag=item->type();
-     ADM_assert(itag>=ACTIVE_FILTER_BASE);
-     itag-=ACTIVE_FILTER_BASE;
-     /* Filter 0 is the decoder ...*/
-     ADM_info("Rank : %d\n",itag);
-
-   //   ADM_assert(itag);
-     /**/
-        ADM_vf_configureFilterAtIndex(itag);
-        buildActiveFilterList ();
-		setSelected(itag);
+    int itag=getTagForActiveSelection();
+    if(-1==itag)
+        return;
+    ADM_vf_configureFilterAtIndex(itag);
+    buildActiveFilterList ();
+    setSelected(itag);
 }
 /**
         \fn     up( bool b)
         \brief  Move selected filter one place up
 */
-void filtermainWindow::up( bool b)
+void filtermainWindow::moveUp( )
 {
-   QListWidgetItem *item=activeList->currentItem();
-   if(!item)
-   {
-      ADM_warning("No selection\n");
-      return;
-   }
-
-     int itag=item->type();
-     ADM_assert(itag>=ACTIVE_FILTER_BASE);
-     itag-=ACTIVE_FILTER_BASE;
-      ADM_info("Rank : %d\n",itag);
-
-        if (!itag ) return;
-        ADM_vf_moveFilterUp(itag);
-        buildActiveFilterList ();
-        setSelected(itag-1);
+    int itag=getTagForActiveSelection();
+    if(-1==itag)
+        return;
+    ADM_vf_moveFilterUp(itag);
+    buildActiveFilterList ();
+    setSelected(itag-1);
 }
 /**
         \fn     down( bool b)
         \brief  Move selected filter one place down
 */
-void filtermainWindow::down( bool b)
+void filtermainWindow::moveDown( )
 {
-   QListWidgetItem *item=activeList->currentItem();
-   if(!item)
-   {
-      ADM_warning("No selection\n");
-      return;
-   }
-
-     int itag=item->type();
-     ADM_assert(itag>=ACTIVE_FILTER_BASE);
-     itag-=ACTIVE_FILTER_BASE;
-     /* Filter 0 is the decoder ...*/
-      ADM_info("Rank : %d\n",itag);
-     //ADM_assert(itag);
-
+    int itag=getTagForActiveSelection();
+    if(-1==itag)
+        return;
     if (((int) itag < (int) (nb_active_filter - 1)))
-        {
-            ADM_vf_moveFilterDown(itag);
-            buildActiveFilterList ();
-            setSelected(itag+1);
-        }
+    {
+        ADM_vf_moveFilterDown(itag);
+        buildActiveFilterList ();
+        setSelected(itag+1);
+    }
 }
 /**
         \fn     filtermainWindow::filterFamilyClick( QListWidgetItem  *item)
@@ -341,18 +333,20 @@ void filtermainWindow::filterFamilyClick(QListWidgetItem *item)
 void filtermainWindow::filterFamilyClick(int  m)
 {
 
-        if(m>=0)
-                displayFamily(m);
+    if(m>=0)
+            displayFamily(m);
 
 }
+/**
+ * \fn displayFamily
+ * @param family
+ */
 void filtermainWindow::displayFamily(uint32_t family)
 {
-
-
   ADM_assert(family<VF_MAX);
 
   uint32_t nb=ADM_vf_getNbFiltersInCategory((VF_CATEGORY)family);
-    ADM_info("Video filter Family :%u, nb %d\n",family,nb);
+  ADM_info("Video filter Family :%u, nb %d\n",family,nb);
   QSize sz;
   availableList->clear();
   for (uint32_t i = 0; i < nb; i++)
@@ -369,7 +363,7 @@ void filtermainWindow::displayFamily(uint32_t family)
      }
 
   if (nb)
-	  availableList->setCurrentRow(0);
+    availableList->setCurrentRow(0);
 }
 
 /**
@@ -378,7 +372,6 @@ void filtermainWindow::displayFamily(uint32_t family)
 */
 void filtermainWindow::activeDoubleClick( QListWidgetItem  *item)
 {
-
     configure(0);
 }
 /**
@@ -387,60 +380,9 @@ void filtermainWindow::activeDoubleClick( QListWidgetItem  *item)
 */
 void filtermainWindow::allDoubleClick( QListWidgetItem  *item)
 {
-
     add(0);
 }
-/**
-        \fn     filtermainWindow::partial( bool b)
-        \brief  Partialize one filter
-*/
-void filtermainWindow::partial( bool b)
-{
-#if 0
-  printf("partial\n");
-   QListWidgetItem *item=activeList->currentItem();
-   if(!item)
-   {
-      printf("No selection\n");
-      return;
-   }
 
-     int itag=item->type();
-     ADM_assert(itag>=ACTIVE_FILTER_BASE);
-     itag-=ACTIVE_FILTER_BASE;
-     /* Filter 0 is the decoder ...*/
-      ADM_info("Rank : %d\n",itag);
-      //ADM_assert(itag);
-
-        AVDMGenericVideoStream *replace;
-        CONFcouple *conf;
-        conf = videofilters[itag].conf;
-        if (videofilters[itag].tag == VF_PARTIAL_FILTER)	// cannot recurse
-        {
-            GUI_Error_HIG (QT_TRANSLATE_NOOP("adm","The filter is already partial"), NULL);
-            return;
-        }
-
-        replace =new ADMVideoPartial (videofilters[itag - 1].
-                                      filter,
-                                      videofilters[itag].tag,
-                                      conf);
-
-        if(replace->configure (videofilters[itag - 1].filter))
-        {
-            delete videofilters[itag].filter;
-            if (conf) delete conf;
-            videofilters[itag].filter = replace;
-            replace->getCoupledConf (&conf);
-            videofilters[itag].conf = conf;
-            videofilters[itag].tag = VF_PARTIAL_FILTER;
-            getFirstVideoFilter ();
-            buildActiveFilterList ();
-			setSelected(itag);
-        }
-        else delete replace;
-#endif
-}
 /**
         \fn setup
         \brief Prepare
@@ -457,35 +399,21 @@ void filtermainWindow::setupFilters(void)
 void filtermainWindow::buildActiveFilterList(void)
 {
 
-	activeList->clear();
+    activeList->clear();
     int nb=ADM_vf_getSize();
     printf("%d active filters\n",nb);
-	for (uint32_t i = 0; i < nb; i++)
-	{
-            uint32_t                instanceTag=ADM_vf_getTag(i);
-            ADM_coreVideoFilter     *instance=ADM_vf_getInstance(i);
-            const char *name= ADM_vf_getDisplayNameFromTag(instanceTag);
-            const char *conf=instance->getConfiguration();
-            printf("%d %s\n",i,name);
-#if 0
-		const char *name = instance->;
-		const char *conf = videofilters[i].filter->printConf ();
-		int namelen = strlen (name);
-
-		while (*conf == ' ')
-			++conf;
-
-		if (strncasecmp (name, conf, namelen) == 0)
-		{
-			conf += namelen;
-			while (*conf == ' ' || *conf == ':')
-				++conf;
-		}
-#endif
-		QString str = QString("<b>") + name + QString("</b><br>\n<small>") + conf + QString("</small>");
-		QListWidgetItem *item=new QListWidgetItem(str,activeList,ACTIVE_FILTER_BASE+i);
-		activeList->addItem(item);
-	}
+    for (uint32_t i = 0; i < nb; i++)
+    {
+        uint32_t                instanceTag=ADM_vf_getTag(i);
+        ADM_coreVideoFilter     *instance=ADM_vf_getInstance(i);
+        const char *name= ADM_vf_getDisplayNameFromTag(instanceTag);
+        const char *conf=instance->getConfiguration();
+        printf("%d %s\n",i,name);
+        QString str = QString("<b>") + name + QString("</b><br>\n<small>") + conf + QString("</small>");
+        QListWidgetItem *item=new QListWidgetItem(str,activeList,ACTIVE_FILTER_BASE+i);
+        printf("Active item :%p\n",item);
+        activeList->addItem(item);
+    }
 
 }
   /**
@@ -498,6 +426,19 @@ filtermainWindow::filtermainWindow(QWidget* parent) : QDialog(parent)
 
     availableList=ui.listWidgetAvailable;
     activeList=ui.listWidgetActive;
+    
+    printf("active : %p\n",activeList);
+    
+#if 1 //def NO_DELEGATE    
+    activeList->setSelectionMode(QAbstractItemView::SingleSelection);
+    activeList->setDragEnabled(true);
+    activeList->setDragDropMode(QAbstractItemView::InternalMove);
+    activeList->setDropIndicatorShown(true);
+    activeList->viewport()->setAcceptDrops(true);
+    connect(activeList,SIGNAL(indexesMoved(const QModelIndexList &)),this,SLOT(indexesMoved(const QModelIndexList &)));
+#endif    
+    activeList->setItemDelegate(new FilterItemDelegate(activeList));
+    
     connect(ui.listFilterCategory,SIGNAL(itemDoubleClicked(QListWidgetItem *)),
                 this,SLOT(filterFamilyClick(QListWidgetItem *)));
     connect(ui.listFilterCategory,SIGNAL(itemClicked(QListWidgetItem *)),
@@ -506,49 +447,43 @@ filtermainWindow::filtermainWindow(QWidget* parent) : QDialog(parent)
     connect(activeList,SIGNAL(itemDoubleClicked(QListWidgetItem *)),this,SLOT(activeDoubleClick(QListWidgetItem *)));
     connect(availableList,SIGNAL(itemDoubleClicked(QListWidgetItem *)),this,SLOT(allDoubleClick(QListWidgetItem *)));
 
-    connect((ui.toolButtonConfigure),SIGNAL(clicked(bool)),this,SLOT(configure(bool)));
-    connect((ui.toolButtonAdd),SIGNAL(clicked(bool)),this,SLOT(add(bool)));
-    connect((ui.pushButtonRemove),SIGNAL(clicked(bool)),this,SLOT(remove(bool)));
-    connect((ui.toolButtonUp),SIGNAL(clicked(bool)),this,SLOT(up(bool)));
-    connect((ui.toolButtonDown),SIGNAL(clicked(bool)),this,SLOT(down(bool)));
-    connect((ui.toolButtonPartial),SIGNAL(clicked(bool)),this,SLOT(partial(bool)));
     connect(ui.buttonClose, SIGNAL(clicked(bool)), this, SLOT(accept()));
-    connect(ui.pushLoad, SIGNAL(clicked(bool)), this, SLOT(loadFilters(bool)));
-    connect(ui.pushSave, SIGNAL(clicked(bool)), this, SLOT(saveFilters(bool)));
-#if 0
-    connect(ui.pushButtonDVD, SIGNAL(clicked(bool)), this, SLOT(DVD(bool)));
-    connect(ui.pushButtonVCD, SIGNAL(clicked(bool)), this, SLOT(VCD(bool)));
-    connect(ui.pushButtonSVCD, SIGNAL(clicked(bool)), this, SLOT(SVCD(bool)));
-    connect(ui.pushButtonHalfDVD, SIGNAL(clicked(bool)), this, SLOT(halfD1(bool)));
-#endif
-	connect(ui.pushButtonPreview, SIGNAL(clicked(bool)), this, SLOT(preview(bool)));
+    connect(ui.pushButtonPreview, SIGNAL(clicked(bool)), this, SLOT(preview(bool)));
 
-	activeList->setItemDelegate(new FilterItemDelegate(activeList));
-	availableList->setItemDelegate(new FilterItemDelegate(availableList));
+
+    availableList->setItemDelegate(new FilterItemDelegate(availableList));
 
     displayFamily(0);
     buildActiveFilterList();
-	setSelected(nb_active_filter - 1);
+    setSelected(nb_active_filter - 1);
 
-	previewDialog = NULL;
-	previewDialogX = INT_MIN;
-	previewDialogY = INT_MIN;
+    previewDialog = NULL;
+    previewDialogX = INT_MIN;
+    previewDialogY = INT_MIN;
     //____________________
     //  Context Menu
     //____________________
     QAction *add = new  QAction(QString("Add"),this);
     availableList->setContextMenuPolicy(Qt::ActionsContextMenu);
     availableList->addAction(add );
-    connect(add,SIGNAL(activated()),this,SLOT(add()));
-
-	//previewFrameIndex = curframe;
+    connect(add,SIGNAL(activated()),this,SLOT(addSlot()));
+    
+    
     QAction *remove = new  QAction(QString("Remove"),this);
     QAction *configure = new  QAction(QString("Configure"),this);
+    QAction *up = new  QAction(QString("Move up"),this);
+    QAction *down = new  QAction(QString("Move down"),this);
+    
     activeList->setContextMenuPolicy(Qt::ActionsContextMenu);
-    activeList->addAction(remove);
+    activeList->addAction(up);
+    activeList->addAction(down);
     activeList->addAction(configure);
-    connect(remove,SIGNAL(activated()),this,SLOT(remove()));
-    connect(configure,SIGNAL(activated()),this,SLOT(configure()));
+    activeList->addAction(remove);
+    
+    connect(remove,SIGNAL(triggered()),this,SLOT(removeAction()));
+    connect(configure,SIGNAL(triggered()),this,SLOT(configureAction()));
+    connect(up,SIGNAL(triggered()),this,SLOT(moveUp()));
+    connect(down,SIGNAL(triggered()),this,SLOT(moveDown()));
 
  }
 /**
@@ -556,96 +491,52 @@ filtermainWindow::filtermainWindow(QWidget* parent) : QDialog(parent)
 */
 filtermainWindow::~filtermainWindow()
 {
-    if(previewDialog) delete previewDialog;
+    if(previewDialog) 
+        delete previewDialog;
     previewDialog=NULL;
 
 }
-/*******************************************************/
-
-int GUI_handleVFilter(void);
-static void updateFilterList (filtermainWindow *dialog);
-
-/**
-      \fn     GUI_handleVFilter(void)
-      \brief  Show the main filter window allowing user to add/remove/configure video filters
-
-
-*/
-int GUI_handleVFilter(void)
-{
-	filtermainWindow dialog(qtLastRegisteredDialog());
-	qtRegisterDialog(&dialog);
-
-	dialog.exec();
-
-	qtUnregisterDialog(&dialog);
-
-	return 0;
-}
-/**
-    \fn partialCb
-    \brief Partial callback to configure the swallowed filter
-
-*/
-static void partialCb(void *cookie);
-void partialCb(void *cookie)
-{
-#if 0
-  void **params=(void **)cookie;
-  AVDMGenericVideoStream *son=(AVDMGenericVideoStream *)params[0];
-  AVDMGenericVideoStream *previous=(AVDMGenericVideoStream *)params[1];
-  son->configure(previous);
-#endif
-}
-/**
-    \fn DIA_getPartial
-    \brief Partial dialog
-
-*/
-#if 0
-uint8_t DIA_getPartial(PARTIAL_CONFIG *param,AVDMGenericVideoStream *son,AVDMGenericVideoStream *previous)
-{
-
-#define PX(x) &(param->x)
-  void *params[2]={son,previous};
-         uint32_t fmax=previous->getInfo()->nb_frames;
-         if(fmax) fmax--;
-
-         diaElemUInteger  start(PX(_start),QT_TRANSLATE_NOOP("adm","Partial Start Frame:"),0,fmax);
-         diaElemUInteger  end(PX(_end),QT_TRANSLATE_NOOP("adm","Partial End Frame:"),0,fmax);
-         diaElemButton    button(QT_TRANSLATE_NOOP("adm","Configure child"), partialCb,params);
-
-         diaElem *tabs[]={&start,&end,&button};
-        return diaFactoryRun(QT_TRANSLATE_NOOP("adm","Partial Video Filter"),3,tabs);
-
-
-}
-#endif
 
 /**
     \fn    Add
     \brief Right click on an available filer
 */
-void filtermainWindow::add(void)
+void filtermainWindow::addSlot(void)
 {
     add(true);
 }
 /**
-    \fn    Add
-    \brief Right click on an available filer
-*/
-void filtermainWindow::remove(void)
+ * 
+ * @return 
+ */
+void filtermainWindow::indexesMoved(const QModelIndexList & indexes)
 {
-    remove(true);
+    int n=indexes.size();
+    printf(" -- Moved with %d inputs\n",n);
+    for(int i=0;i<n;i++)
+    {
+        //indexes[i].
+        
+    }
 }
+/*******************************************************/
+
+int GUI_handleVFilter(void);
 /**
-    \fn    Add
-    \brief Right click on an available filer
+      \fn     GUI_handleVFilter(void)
+      \brief  Show the main filter window allowing user to add/remove/configure video filters
 */
-void filtermainWindow::configure(void)
+int GUI_handleVFilter(void)
 {
-    configure(true);
+    filtermainWindow dialog(qtLastRegisteredDialog());
+    qtRegisterDialog(&dialog);
+    dialog.exec();
+    qtUnregisterDialog(&dialog);
+    return 0;
 }
+//EOF
+#if 0
+
 /**
  * \fn loadFilters
  * \brief load a filter from .py
@@ -657,7 +548,7 @@ void filtermainWindow::loadFilters( bool b)
     printf("Load filters\n");
 
     char name[1024];
-    
+
     if(!FileSel_SelectRead(QT_TR_NOOP("Load video filters.."),name,1023,NULL))
         return;
     call_scriptEngine(name);
@@ -672,13 +563,13 @@ void filtermainWindow::saveFilters( bool b)
 {
     printf("save filters\n");
     char name[1024];
-    
+
     if(!FileSel_SelectWrite(QT_TR_NOOP("Save video filters.."),name,1023,NULL))
         return;
     printf("save filters, part 2\n");
     IScriptEngine *engine=getPythonScriptEngine();
     IScriptWriter *writer = engine->createScriptWriter();
-    
+
     ADM_ScriptGenerator generator(video_body, writer);
     std::stringstream stream(std::stringstream::in | std::stringstream::out);
     std::string fileName = std::string(name);
@@ -700,5 +591,13 @@ void filtermainWindow::saveFilters( bool b)
     ADM_fclose(file);
 }
 
-//EOF
+/**
+ * // Stubb
+ * @param b
+ */
+void filtermainWindow::partial(bool b)
+{
+    
+}
+#endif
 
