@@ -18,7 +18,7 @@
 #include <QKeyEvent>
 #include <QGraphicsView>
 #include <QtCore/QDir>
-
+#include <QMessageBox>
 #include "ADM_cpp.h"
 #define MENU_DECLARE
 #include "Q_gui2.h"
@@ -39,8 +39,16 @@
 #include "config.h"
 #include "ADM_preview.h"
 #include "DIA_defaultAskAvisynthPort.hxx"
+#include "ADM_systemTrayProgress.h"
 using namespace std;
 
+#define ADM_SLIDER_REFRESH_PERIOD 500
+
+
+#if defined(USE_SDL) && ( !defined(_WIN32) && !defined(__APPLE__))
+    #define SDL_ON_LINUX
+#endif
+
 #ifdef USE_OPENGL
 extern bool ADM_glHasActiveTexture(void);
 void UI_Qt4InitGl(void);
@@ -52,7 +60,7 @@ extern int global_argc;
 extern char **global_argv;
 
 extern int automation(void );
-extern void HandleAction(Action a);
+extern void sendAction(Action a);
 extern int encoderGetEncoderCount (void);
 extern const char *encoderGetIndexedName (uint32_t i);
 uint32_t audioEncoderGetNumberOfEncoders(void);
@@ -63,8 +71,8 @@ extern void loadTranslator(void);
 extern void initTranslator(void);
 extern void destroyTranslator(void);
 extern ADM_RENDER_TYPE UI_getPreferredRender(void);
-extern int A_openAvi(const char *name);
-extern int A_appendAvi(const char *name);
+extern int A_openVideo(const char *name);
+extern int A_appendVideo(const char *name);
 
 int SliderIsShifted=0;
 static void setupMenus(void);
@@ -77,6 +85,8 @@ static int currentFrame = 0;
 bool     ADM_ve6_getEncoderInfo(int filter, const char **name, uint32_t *major,uint32_t *minor,uint32_t *patch);
 uint32_t ADM_ve6_getNbEncoders(void);
 void UI_refreshCustomMenu(void);
+admUITaskBarProgress *QuiTaskBarProgress;
+extern admUITaskBarProgress *createADMTaskBarProgress();
 QWidget *QuiMainWindows=NULL;
 QWidget *VuMeter=NULL;
 QGraphicsView *drawWindow=NULL;
@@ -142,7 +152,7 @@ void MainWindow::comboChanged(int z)
 		}
 		ui.pushButtonVideoConf->setEnabled(b);
 		ui.pushButtonVideoFilter->setEnabled(b);
-		HandleAction (ACT_VIDEO_CODEC_CHANGED) ;
+		sendAction (ACT_VIDEO_CODEC_CHANGED) ;
 	}
 	else if (obj == ui.comboBoxAudio)
 	{
@@ -153,26 +163,82 @@ void MainWindow::comboChanged(int z)
 		}
 		ui.pushButtonAudioConf->setEnabled(b);
 		ui.pushButtonAudioFilter->setEnabled(b);
-		HandleAction (ACT_AUDIO_CODEC_CHANGED) ;
+		sendAction (ACT_AUDIO_CODEC_CHANGED) ;
 	}
 }
-
+/**
+ * \fn sliderValueChanged
+ * @param u
+ */
 void MainWindow::sliderValueChanged(int u)
 {
-	if(!_upd_in_progres)
-		HandleAction(ACT_Scale);
+  
+    if(_upd_in_progres)
+      return;
+    if(refreshCapEnabled)
+        switch(dragState)
+        { 
+            default:
+            case dragState_Normal:
+                sendAction(ACT_Scale);
+                break;
+            case dragState_Active:
+                dragTimer.stop();
+                dragTimer.start(refreshCapValue);
+                dragState=dragState_HoldOff;
+                break;            
+            case dragState_HoldOff:
+              break;            
+          }
+    else
+         sendAction(ACT_Scale);
 }
-
+/**
+ * \fn dragTimerTimeout
+ */
+void MainWindow::dragTimerTimeout(void)
+{
+  ADM_info("Drag timeout\n");
+  switch(dragState)
+    {
+        default:
+        case dragState_Normal:           
+        case dragState_Active:           
+            break;            
+        case dragState_HoldOff:
+            dragState=dragState_Active;
+            sendAction(ACT_Scale);
+            break;
+    }
+}
+/**
+ * \fn sliderMoved
+ */
 void MainWindow::sliderMoved(int value)
 {
+    //ADM_info("Moved\n");
 	SliderIsShifted = shiftKeyHeld;
 }
-
+/**
+ * \fn sliderReleased
+ */
 void MainWindow::sliderReleased(void)
 {
+  //ADM_info("Released\n");
 	SliderIsShifted = 0;
+    dragTimer.stop();
+    dragState=dragState_Normal;
+    sendAction(ACT_Scale);
+}
+/**
+ * \fn sliderPressed
+ */
+void MainWindow::sliderPressed(void)
+{ 
+  dragTimer.stop();
+  dragState=dragState_Active;
+//  ADM_info("Pressed\n");
 }
-
 void MainWindow::thumbSlider_valueEmitted(int value)
 {
         if (value > 0)
@@ -204,7 +270,7 @@ void MainWindow::audioToggled(bool checked)
 
 void MainWindow::previewModeChanged(int  flop)
 {
-	HandleAction(ACT_PreviewChanged);
+	sendAction(ACT_PreviewChanged);
 }
 
 void MainWindow::timeChangeFinished(void)
@@ -214,14 +280,14 @@ void MainWindow::timeChangeFinished(void)
 
 void MainWindow::currentFrameChanged(void)
 {
-//	HandleAction(ACT_JumpToFrame);
+//	sendAction(ACT_JumpToFrame);
 
 	this->setFocus(Qt::OtherFocusReason);
 }
 
 void MainWindow::currentTimeChanged(void)
 {
-	HandleAction(ACT_GotoTime);
+	sendAction(ACT_GotoTime);
 
 	this->setFocus(Qt::OtherFocusReason);
 }
@@ -233,13 +299,19 @@ MainWindow::MainWindow(const vector<IScriptEngine*>& scriptEngines) : _scriptEng
 {
 	qtRegisterDialog(this);
 	ui.setupUi(this);
+        dragState=dragState_Normal;
+        refreshCapEnabled=false;
+        prefs->get(FEATURES_CAP_REFRESH_ENABLED,&refreshCapEnabled);
+        prefs->get(FEATURES_CAP_REFRESH_VALUE,&refreshCapValue);
 
 #if defined(__APPLE__) && defined(USE_SDL)
-	ui.actionAbout_avidemux->setMenuRole(QAction::NoRole);
-	ui.actionPreferences->setMenuRole(QAction::NoRole);
-	ui.actionQuit->setMenuRole(QAction::NoRole);
+	//ui.actionAbout_avidemux->setMenuRole(QAction::NoRole);
+	//ui.actionPreferences->setMenuRole(QAction::NoRole);
+	//ui.actionQuit->setMenuRole(QAction::NoRole);
 #endif
-
+        //
+        connect( this,SIGNAL(actionSignal(Action )),this,SLOT(actionSlot(Action )));
+        //
         connect( ui.checkDisplayOut,SIGNAL(stateChanged(int)),this,SLOT(previewModeChanged(int)));
 
 	/*
@@ -257,9 +329,16 @@ MainWindow::MainWindow(const vector<IScriptEngine*>& scriptEngines) : _scriptEng
 	slider=ui.horizontalSlider;
 	slider->setMinimum(0);
 	slider->setMaximum(1000000000);
+#if 1
+        slider->setTickInterval(10000000);
+        slider->setTickPosition(QSlider::TicksBothSides);
+#endif
 	connect( slider,SIGNAL(valueChanged(int)),this,SLOT(sliderValueChanged(int)));
 	connect( slider,SIGNAL(sliderMoved(int)),this,SLOT(sliderMoved(int)));
 	connect( slider,SIGNAL(sliderReleased()),this,SLOT(sliderReleased()));
+    connect( slider,SIGNAL(sliderPressed()),this,SLOT(sliderPressed()));
+    connect( &dragTimer, SIGNAL(timeout()), this, SLOT(dragTimerTimeout()));
+    
 
    // Thumb slider
     ui.sliderPlaceHolder->installEventFilter(this);
@@ -336,6 +415,7 @@ MainWindow::MainWindow(const vector<IScriptEngine*>& scriptEngines) : _scriptEng
 	ui.audioMetreWidget->setTitleBarWidget(dummy4);
 
 	this->adjustSize();
+        QuiTaskBarProgress=createADMTaskBarProgress();
 }
 /**
     \fn searchToolBar
@@ -365,7 +445,7 @@ void MainWindow::searchToolBar(QAction *action)
         {
             if(!strcmp(name,t->name))
             {
-                HandleAction(t->event);
+                sendAction(t->event);
                 ADM_dealloc( name);
                 return;
             }
@@ -477,7 +557,7 @@ void MainWindow::checkChanged(int state)
 */
 void MainWindow::timeChanged(int)
 {
-	HandleAction (ACT_TimeShift) ;
+	sendAction (ACT_TimeShift) ;
 }
 /**
     \fn searchMenu
@@ -489,7 +569,7 @@ void MainWindow::searchMenu(QAction * action,MenuEntry *menu, int nb)
         MenuEntry *m=menu+i;
         if(m->cookie==(void*)action)
         {
-            HandleAction (m->event);
+            sendAction (m->event);
         }
     }
 }
@@ -527,14 +607,14 @@ void MainWindow::buttonPressed(void)
 
 
 	if(action!=ACT_DUMMY)
-		HandleAction (action);
+		sendAction (action);
 
 }
 void MainWindow::toolButtonPressed(bool i)
 {
 	buttonPressed();
 }
-
+#ifdef ENABLE_EVENT_FILTER
 bool MainWindow::eventFilter(QObject* watched, QEvent* event)
 {
 	QKeyEvent *keyEvent;
@@ -550,31 +630,31 @@ bool MainWindow::eventFilter(QObject* watched, QEvent* event)
 				{
 					case Qt::Key_Left:
 						if (keyEvent->modifiers() == Qt::ShiftModifier)
-							HandleAction(ACT_Back1Second);
+							sendAction(ACT_Back1Second);
 						else if (keyEvent->modifiers() == Qt::ControlModifier)
-							HandleAction(ACT_Back2Seconds);
+							sendAction(ACT_Back2Seconds);
 						else if (keyEvent->modifiers() == (Qt::ControlModifier | Qt::ShiftModifier))
-							HandleAction(ACT_Back4Seconds);
+							sendAction(ACT_Back4Seconds);
 						else
-							HandleAction(ACT_PreviousFrame);
+							sendAction(ACT_PreviousFrame);
 
 						return true;
 					case Qt::Key_Right:
 						if (keyEvent->modifiers() == Qt::ShiftModifier) 
-							HandleAction(ACT_Forward1Second);
+							sendAction(ACT_Forward1Second);
 						else if (keyEvent->modifiers() == Qt::ControlModifier) 
-							HandleAction(ACT_Forward2Seconds);
+							sendAction(ACT_Forward2Seconds);
 						else if (keyEvent->modifiers() == (Qt::ControlModifier | Qt::ShiftModifier)) 
-							HandleAction(ACT_Forward4Seconds);
+							sendAction(ACT_Forward4Seconds);
 						else 
-							HandleAction(ACT_NextFrame);
+							sendAction(ACT_NextFrame);
 
 						return true;
 					case Qt::Key_Up:
-						HandleAction(ACT_NextKFrame);
+						sendAction(ACT_NextKFrame);
 						return true;
 					case Qt::Key_Down:
-						HandleAction(ACT_PreviousKFrame);
+						sendAction(ACT_PreviousKFrame);
 						return true;
 					case Qt::Key_Shift:
 						shiftKeyHeld = 1;
@@ -582,18 +662,18 @@ bool MainWindow::eventFilter(QObject* watched, QEvent* event)
 
 					case Qt::Key_BracketLeft:
                                                 if (keyEvent->modifiers() == Qt::ControlModifier)
-                                                        HandleAction(ACT_GotoMarkA);
+                                                        sendAction(ACT_GotoMarkA);
                                                 return true;
 
 					case Qt::Key_BracketRight:
 						if (keyEvent->modifiers() == Qt::ControlModifier)
-							HandleAction(ACT_GotoMarkB);
+							sendAction(ACT_GotoMarkB);
 						return true;
 				}
 			}
 			/* else */ if (keyEvent->key() == Qt::Key_Space)
 			{
-				HandleAction(ACT_PlayAvi);
+				sendAction(ACT_PlayAvi);
 				return true;
 			}
 
@@ -620,7 +700,7 @@ bool MainWindow::eventFilter(QObject* watched, QEvent* event)
 
 	return QObject::eventFilter(watched, event);
 }
-
+#endif
 void MainWindow::mousePressEvent(QMouseEvent* event)
 {
 	this->setFocus(Qt::OtherFocusReason);
@@ -654,9 +734,9 @@ void MainWindow::openFiles(QList<QUrl> urlList)
 		if (info.isFile())
 		{
 			if (avifileinfo)
-				A_appendAvi(fileName.toUtf8().constData());
+				A_appendVideo(fileName.toUtf8().constData());
 			else
-				A_openAvi(fileName.toUtf8().constData());
+				A_openVideo(fileName.toUtf8().constData());
 		}
 	}
 }
@@ -666,7 +746,7 @@ void MainWindow::previousIntraFrame(void)
 	if (ui.spinBox_TimeValue->hasFocus())
 		ui.spinBox_TimeValue->stepDown();
 	else
-		HandleAction(ACT_PreviousKFrame);
+		sendAction(ACT_PreviousKFrame);
 }
 
 void MainWindow::nextIntraFrame(void)
@@ -674,7 +754,7 @@ void MainWindow::nextIntraFrame(void)
 	if (ui.spinBox_TimeValue->hasFocus())
 		ui.spinBox_TimeValue->stepUp();
 	else
-		HandleAction(ACT_NextKFrame);
+		sendAction(ACT_NextKFrame);
 }
 
 MainWindow::~MainWindow()
@@ -696,6 +776,19 @@ static const UI_FUNCTIONS_T UI_Hooks=
 
 
 static myQApplication *myApplication=NULL;
+/**
+ * \fn UI_reset
+ * \brief reset
+ * @return 
+ */
+bool  	UI_reset(void)
+{
+    UI_setVideoCodec(0);
+    UI_setAudioCodec(0);
+    UI_setCurrentPreview(false);
+    return true;
+}
+
 /**
     \fn  UI_Init
     \brief First part of UI initialization
@@ -712,15 +805,16 @@ int UI_Init(int nargc, char **nargv)
         Q_INIT_RESOURCE(avidemux);
 	Q_INIT_RESOURCE(filter);
 
-	myApplication=new myQApplication (global_argc, global_argv);
 #if defined(__APPLE__)
  printf("Setting qt plugin folder\n");
  QDir dir(QApplication::applicationDirPath());
  dir.cdUp();
  dir.cdUp();
  dir.cd("plugins");
+ printf("New plugin path =%s\n",dir.absolutePath().toUtf8().constData());
  QApplication::setLibraryPaths(QStringList(dir.absolutePath()));
 #endif
+	myApplication=new myQApplication (global_argc, global_argv);
 	myApplication->connect(myApplication, SIGNAL(lastWindowClosed()), myApplication, SLOT(quit()));
 
 	loadTranslator();
@@ -730,20 +824,22 @@ int UI_Init(int nargc, char **nargv)
 
 uint8_t initGUI(const vector<IScriptEngine*>& scriptEngines)
 {
-	MainWindow *mw = new MainWindow(scriptEngines);
-	mw->show();
+    MainWindow *mw = new MainWindow(scriptEngines);
+    mw->show();
 
-	QuiMainWindows = (QWidget*)mw;
+    QuiMainWindows = (QWidget*)mw;
 
-	uint32_t w, h;
+    uint32_t w, h;
 
-	UI_getPhysicalScreenSize(QuiMainWindows, &w,&h);
-	printf("The screen seems to be %u x %u px\n",w,h);
+    UI_getPhysicalScreenSize(QuiMainWindows, &w,&h);
+    printf("The screen seems to be %u x %u px\n",w,h);
+    mw->ui.frame_video->setAttribute(Qt::WA_OpaquePaintEvent);
+    
 
-	UI_QT4VideoWidget(mw->ui.frame_video);  // Add the widget that will handle video display
-        mw->ui.frame_video->setAcceptDrops(true); // needed for drag and drop to work on windows
-	UI_updateRecentMenu();
-	UI_updateRecentProjectMenu();
+    UI_QT4VideoWidget(mw->ui.frame_video);  // Add the widget that will handle video display
+    mw->ui.frame_video->setAcceptDrops(true); // needed for drag and drop to work on windows
+    UI_updateRecentMenu();
+    UI_updateRecentProjectMenu();
 
     // Init vumeter
     VuMeter=mw->ui.frameVU;
@@ -790,8 +886,7 @@ void destroyGUI(void)
 }
 void callBackQtWindowDestroyed()
 {
-	
-
+    
 }
 /**
 
@@ -827,14 +922,24 @@ int UI_getCurrentPreview(void)
 */
 void UI_setCurrentPreview(int ne)
 {
-        WIDGET(checkDisplayOut)->setChecked(true);
+        WIDGET(checkDisplayOut)->setChecked(!!ne);
 }
 /**
         \fn FatalFunctionQt
 */
 static void FatalFunctionQt(const char *title, const char *info)
 {
-	GUI_Info_HIG(ADM_LOG_IMPORTANT, title, info);
+    printf("Crash Dump for %s\n",title);
+    printf("%s\n",info);
+    fflush(stdout);
+    
+    QMessageBox msgBox;
+    msgBox.setText(title);
+    msgBox.setInformativeText(QT_TR_NOOP("The application has encountered a fatal problem\nThe current editing has been saved and will be reloaded at next start"));
+    msgBox.setDetailedText(info);
+    msgBox.setIcon(QMessageBox::Critical);
+    msgBox.exec();
+    abort();
 }
 
 /**
@@ -845,6 +950,7 @@ int UI_RunApp(void)
 {
     uiRunning=true;
     setupMenus();
+    QuiTaskBarProgress->setParent(QuiMainWindows);
     ADM_setCrashHook(&saveCrashProject, &FatalFunctionQt);
 	
           
@@ -1051,7 +1157,14 @@ void UI_setFrameType( uint32_t frametype,uint32_t qp)
 	WIDGET(label_8)->setText(string);
 
 }
-
+/**
+ * 
+ * @return 
+ */
+admUITaskBarProgress *UI_getTaskBarProgress()
+{
+    return QuiTaskBarProgress;
+}
 /**
     \fn     UI_updateFrameCount(uint32_t curFrame)
     \brief  Display the current displayed frame #
@@ -1251,6 +1364,16 @@ bool UI_setDecoderName(const char *name)
     WIDGET(labelVideoDecoder)->setText(name);
     return true;
 }
+/**
+ * \fn UI_setDisplayName
+ * \brief display current displayEngine name
+ */
+bool UI_setDisplayName(const char *name)
+{
+    WIDGET(labelDisplay)->setText(name);
+    return true;
+}
+
 /**
     \fn UI_hasOpengl
 */
@@ -1294,6 +1417,9 @@ void UI_setAudioTrackCount( int nb )
 /**
  * \fn dtor
  */
+#ifdef SDL_ON_LINUX
+extern void disableExitHandler();
+#endif
 myQApplication::~myQApplication()
 {
     
@@ -1313,6 +1439,12 @@ myQApplication::~myQApplication()
   #endif
 #endif
 
+    
+#ifdef SDL_ON_LINUX
+    ADM_warning("This is SDL on linux, exiting brutally to avoid lock.\n");
+    disableExitHandler();
+    ::exit(0); // 
+#endif    
     ADM_warning("Exiting app\n");
 }
                 
