@@ -17,6 +17,7 @@
 #include "IScriptEngine.h"
 #include "prefs.h"
 
+#define ENABLE_EVENT_FILTER
 
 /**
  * \class myQApplication
@@ -58,7 +59,15 @@ class myQApplication : public QApplication
                                 automation();
                 }
 };
-
+/**
+ * \enum ADM_dragState
+ */
+enum ADM_dragState
+{
+    dragState_Normal=0, // no drag
+    dragState_Active=1, // drag on, we process the refresh
+    dragState_HoldOff=2 // Cdrag off, we ignore the refresh
+};
 
 /**
     \class MainWindow
@@ -84,10 +93,24 @@ protected:
     QMenu *recentFiles;
     QMenu *recentProjects;
     QAction *recentFileAction[NB_LAST_FILES];
-	QAction *recentProjectAction[NB_LAST_FILES];
+    QAction *recentProjectAction[NB_LAST_FILES];
     ThumbSlider *thumbSlider;
+    
+    bool     refreshCapEnabled;
+    uint32_t refreshCapValue;
+
 
 public slots:
+        void dragTimerTimeout(void);
+        void actionSlot(Action a)
+        {
+            HandleAction(a);
+        }
+        void sendAction(Action a)
+        {
+            //printf("Sending internal event %d\n",(int)a);
+            emit actionSignal(a);
+        }
 	void timeChanged(int);
         void checkChanged(int);
 	void buttonPressed(void);
@@ -97,6 +120,7 @@ public slots:
 	void sliderValueChanged(int u);
 	void sliderMoved(int value);
 	void sliderReleased(void);
+        void sliderPressed(void);
 	void volumeChange( int u );
 	void audioToggled(bool checked);
 	void previewModeChanged(int status);
@@ -127,11 +151,15 @@ public slots:
         {
                 printf("Close event!\n");
                 //QMainWindow::closeEvent(event);
-                HandleAction(ACT_EXIT);
+                sendAction(ACT_EXIT);
         }
 
+signals:
+        void actionSignal(Action a);
 protected:
-	const std::vector<IScriptEngine*>& _scriptEngines;
+        ADM_dragState    dragState;        
+        QTimer dragTimer;
+	const  std::vector<IScriptEngine*>& _scriptEngines;
 
 	void addScriptDirToMenu(QMenu* scriptMenu, const QString& dir, const QStringList& fileExts);
 	void addScriptEnginesToFileMenu(std::vector<MenuEntry>& fileMenu);
@@ -142,7 +170,9 @@ protected:
 	void buildRecentMenu(QMenu *menu, const char **files, QAction **actions);
         void searchMenu(QAction * action,MenuEntry *menu, int nb);
 	void searchRecentFiles(QAction *action, QAction **actionList, int firstEventId);
+#ifdef   ENABLE_EVENT_FILTER      
 	bool eventFilter(QObject* watched, QEvent* event);
+#endif
 	void mousePressEvent(QMouseEvent* event);
 	void dragEnterEvent(QDragEnterEvent *event);
 	void dropEvent(QDropEvent *event);
