@@ -2,7 +2,7 @@
 /***************************************************************************
     Handle all redraw operation for QT4 
     
-    copyright            : (C) 2006 by mean
+    copyright            : (C) 2006/2015 by mean
     email                : fixounet@free.fr
  ***************************************************************************/
 
@@ -81,19 +81,29 @@ bool ADM_QPreviewCleanup(void)
     rgbDataBuffer=NULL;
     return true;
 }
+/**
+ * 
+ * @param z
+ */
 #ifdef __HAIKU__
 ADM_Qvideo::ADM_Qvideo(QWidget *z) : QWidget(z)  {}
 #else
 ADM_Qvideo::ADM_Qvideo(QWidget *z) : QWidget(z) 
 {    
-setAttribute( Qt::WA_PaintOnScreen, true );
-// Put a transparent background
-//setAutoFillBackground(true);
-QPalette p =  palette();
-QColor color(Qt::black);
-color.setAlpha(0);
-p.setColor( QPalette::Window, color );
-setPalette( p );
+#if QT_VERSION < QT_VERSION_CHECK(5,0,0) 
+    setAttribute( Qt::WA_PaintOnScreen, true );
+   // Put a transparent background
+    //setAutoFillBackground(true);
+    QPalette p =  palette();
+    QColor color(Qt::black);
+    color.setAlpha(0);
+    p.setColor( QPalette::Window, color );
+    setPalette( p );
+   
+#else
+    
+    
+#endif
 
 } //{setAutoFillBackground(false);}
 #endif // Haiku
@@ -101,30 +111,47 @@ ADM_Qvideo::~ADM_Qvideo()
 {
     printf("[Qvideo]Destroying QVideo\n");
     callBackQtWindowDestroyed();
+    
 }
-
+/**
+ * 
+ * @param ev
+ */
 void ADM_Qvideo::paintEvent(QPaintEvent *ev)
-{
-	if (paintEngineType == -1)
-	{
-		QPainter painter(this);
+{	
+    //printf("Draw! \n");
+     if (paintEngineType == -1)
+     {
+        QPainter painter(this);
 
-		if (painter.isActive())
-			paintEngineType = painter.paintEngine()->type();
+        if (painter.isActive())
+                paintEngineType = painter.paintEngine()->type();
 
-		painter.end();
-	}
+        painter.end();
+      }
 
-	if (!displayW || !displayH || !rgbDataBuffer )
-		return;
+    if (!displayW || !displayH || !rgbDataBuffer )
+            return;
 
-	if (renderExposeEventFromUI())
-	{
-		QImage image(rgbDataBuffer,displayW,displayH,QImage::Format_RGB32);
-		QPainter painter(this);
-		painter.drawImage(QPoint(0,0),image);
-		painter.end();
-	}
+    if (renderExposeEventFromUI())
+    {
+            QImage image(rgbDataBuffer,displayW,displayH,QImage::Format_RGB32);
+            QPainter painter(this);
+            if (painter.isActive())
+            {
+                //painter.drawImage(QPoint(0,0),image);
+                const QRect rec=ev->rect();
+                int x=rec.x();
+                int y=rec.y();
+                int w=rec.width();
+                int h=rec.height();
+                painter.drawImage(x,y,image,x,y,w,h);
+                painter.end();
+            }else
+            {
+                printf("Painter inactive!\n");
+            }
+    }
 }
 
 ADM_Qvideo *videoWindow=NULL;
@@ -135,10 +162,6 @@ void UI_QT4VideoWidget(QFrame *host)
    hostFrame=host;
    videoWindow->resize(hostFrame->size());
    videoWindow->show();
-#if 0
-   host->setAutoFillBackground(false);
-   host->setEnabled(false);
-#endif   
    
 }
 //*************************
@@ -208,32 +231,33 @@ void UI_getWindowInfo(void *draw, GUI_WindowInfo *xinfo)
     ADM_assert(videoWindow);
     QWidget* widget = videoWindow->parentWidget();
     xinfo->widget = videoWindow;
+    xinfo->systemWindowId = 0;
 
 #if defined(_WIN32)
 	xinfo->display=(void *)videoWindow->winId();
+        xinfo->systemWindowId=videoWindow->winId();
 #elif defined(__APPLE__)
 	#if defined(ADM_CPU_X86_64)
 		xinfo->display = (void*)videoWindow->winId();
-		xinfo->window = 0;
+                xinfo->systemWindowId=videoWindow->winId();
 	#else
 		xinfo->display = HIViewGetWindow(HIViewRef(widget->winId()));
-		xinfo->window = 0;
+                xinfo->systemWindowId= HIViewGetWindow(HIViewRef(widget->winId()));
 	#endif
 #else
         #if QT_VERSION < QT_VERSION_CHECK(5,0,0) 
                 const QX11Info &info=videoWindow->x11Info();
                 xinfo->display=info.display();
-                // xinfo->window=videoWindow->winId();
-                xinfo->window=videoWindow->winId();
         #else
-                xinfo->window=videoWindow->winId();
                 xinfo->display=XOpenDisplay(NULL);
         #endif
+        xinfo->systemWindowId=videoWindow->winId();
 #endif
-
-	xinfo->x = widget->x();
-	xinfo->y = widget->parentWidget()->height() - (widget->y() + displayH);
-	xinfo->width = displayW;
+        QPoint localPoint(0,0);
+        QPoint windowPoint = videoWindow->mapToGlobal(localPoint);        
+	xinfo->x = windowPoint.x();
+	xinfo->y = windowPoint.y();
+	xinfo->width  = displayW;
 	xinfo->height = displayH;
 }
 
