@@ -22,6 +22,7 @@
 #include "DIA_coreUI_internal.h"
 
 #include "prefs.h"
+#include "ADM_last.h"
 
 namespace ADM_QT4_fileSel
 {
@@ -33,7 +34,7 @@ namespace ADM_QT4_fileSel
 	*/
 static	void GUI_FileSelSelectWriteInternal(const char *label, const char *ext, char **name)
 	{
-		char *tmpname = NULL;
+		
                 *name = NULL;		
                 QString str ;
                 QString fileName,dot=QString(".");
@@ -47,12 +48,31 @@ static	void GUI_FileSelSelectWriteInternal(const char *label, const char *ext, c
                     extSize+=strlen(ext);
                 }
                 //printf("Do filer=%d\n",(int)doFilter);
-                if (prefs->get(LASTFILES_LASTDIR_WRITE,&tmpname))
+                std::string lastFolder;
+                admCoreUtils::getLastWriteFolder(lastFolder);
+                if (lastFolder.size())
 		{
-                        
-                        str = QFileInfo(QString::fromUtf8(tmpname)).path();
+                        QString outputPath = QFileInfo(QString::fromUtf8(lastFolder.c_str())).path();
+
+                        char *tmpinputname = NULL;
+                        QString inputBaseName = QString("");
+                        std::string lastRead;
+                        admCoreUtils::getLastReadFolder(lastRead);
+                        if (lastRead.size())
+                        {
+                            inputBaseName = QFileInfo(QString::fromUtf8(lastRead.c_str())).completeBaseName();
+                        }
+
+                        QString outputExt = QString("");
+                        if (doFilter)
+                        {
+                            outputExt = dot+QString(ext);
+                        }
+                        QString separator = QString("/");
+                        str = outputPath+separator+inputBaseName+outputExt;
+
                 	/* LASTDIR may have gone; then do nothing and use current dir instead (implied) */
-			if (!QDir(str).exists())
+			if (!QDir(outputPath).exists())
                                 str.clear();
 		}
 		
@@ -64,8 +84,8 @@ static	void GUI_FileSelSelectWriteInternal(const char *label, const char *ext, c
                                         label,  // caption
                                         str,    // folder
                                         filterFile,   // filter
-                                        NULL,   // selected filter
-                                        opts);
+                                        NULL,QFileDialog::DontConfirmOverwrite);   // selected filter
+                                        
                 
 		if (fileName.isNull() ) return;
                 
@@ -80,21 +100,21 @@ static	void GUI_FileSelSelectWriteInternal(const char *label, const char *ext, c
                             strcat(*name,"."); strcat(*name,ext);
 
                             fileName=fileName+QString(".")+QString(ext);
-                            QFile newFile(fileName);
-                            if(newFile.exists())
-                            {
-                                QFileInfo fileInfo(newFile);
-                                QString q=QString("Overwrite file ")+fileInfo.fileName()+QString("?");
-                                if(!GUI_Question(q.toUtf8().constData()))
-                                {
-                                    ADM_dezalloc(*name);
-                                    *name=NULL;
-                                    return;
-                                }
-                            }
                         }
                 }
-                prefs->set(LASTFILES_LASTDIR_WRITE, fileName.toUtf8().constData());
+                QFile newFile(fileName);
+                if(newFile.exists())
+                {
+                    QFileInfo fileInfo(newFile);
+                    QString q=QString("Overwrite file ")+fileInfo.fileName()+QString("?");
+                    if(!GUI_Question(q.toUtf8().constData()))
+                    {
+                        ADM_dezalloc(*name);
+                        *name=NULL;
+                        return;
+                    }
+                }
+                admCoreUtils::setLastWriteFolder( std::string(fileName.toUtf8().constData()));
 	}
 
 	
@@ -104,7 +124,7 @@ static	void GUI_FileSelSelectWriteInternal(const char *label, const char *ext, c
 	*/
 static	void GUI_FileSelSelectReadInternal(const char *label, const char *ext, char **name)
 	{
-		char *tmpname = NULL;
+		
                 *name = NULL;		
                 QString str ;
                 QString fileName,dot=QString(".");
@@ -113,10 +133,12 @@ static	void GUI_FileSelSelectReadInternal(const char *label, const char *ext, ch
                 QFileDialog::Options opts;
               
                 //printf("Do filer=%d\n",(int)doFilter);
-                if (prefs->get(LASTFILES_LASTDIR_WRITE,&tmpname))
+                std::string lastFolder;
+                admCoreUtils::getLastReadFolder(lastFolder);
+
+                if (lastFolder.size())
 		{
-                        
-                        str = QFileInfo(QString::fromUtf8(tmpname)).path();
+                        str = QFileInfo(QString::fromUtf8(lastFolder.c_str())).path();
                 	/* LASTDIR may have gone; then do nothing and use current dir instead (implied) */
 			if (!QDir(str).exists())
                                 str.clear();
@@ -136,8 +158,7 @@ static	void GUI_FileSelSelectReadInternal(const char *label, const char *ext, ch
 		if (fileName.isNull() ) return;
                 
                 *name=ADM_strdup(fileName.toUtf8().constData());
-                
-                prefs->set(LASTFILES_LASTDIR_READ, fileName.toUtf8().constData());
+                admCoreUtils::setLastReadFolder(std::string(fileName.toUtf8().constData()));
 	}        
         
         void GUI_FileSelRead(const char *label, char **name)
