@@ -20,7 +20,7 @@
 #include <QtCore/QFileInfo>
 #include <QtCore/QUrl>
 #include <QtGui/QKeyEvent>
-#include <QtGui/QGraphicsView>
+#include <QGraphicsView>
 #include "Q_shell.h"
 #include "ADM_default.h"
 #include "ui_shell.h"
@@ -81,7 +81,7 @@ bool            qShell::evaluate(bool x)
     ui.textBrowser->append(text);
     ui.textBrowser->setFontItalic(false);
     ui.textBrowser_2->setPlainText("");
-    _engine->runScript(text.toAscii().constData(), IScriptEngine::Normal);
+    _engine->runScript(text.toLatin1().constData(), IScriptEngine::Normal);
     return true;
 }
 /**
