@@ -13,6 +13,8 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
+#include <windows.h>
+#include <excpt.h>
 
 #include "ADM_crashdump.h"
 #include <stdio.h>
@@ -20,8 +22,30 @@
 #include <imagehlp.h>
 #include <cxxabi.h>
 
+
+static LONG WINAPI ExceptionFilter(struct _EXCEPTION_POINTERS *exceptionInfo);
 static ADM_saveFunction *mysaveFunction = NULL;
 static ADM_fatalFunction *myFatalFunction = NULL;
+
+
+/**
+ * \fn installSigHandler
+ * \brief add hook to catch exception (null pointers etc...)
+ */
+void installSigHandler(void)
+{    
+    SetUnhandledExceptionFilter(ExceptionFilter);
+}
+/**
+ * \fn uninstallSigHandler
+ * \brief
+ */
+void uninstallSigHandler(void)
+{
+    
+}
+
+
 /**
  * 
  * @param save
@@ -225,7 +249,7 @@ std::string DumpBackTrace(void *processId)
 	RtlCaptureContext_* contextFunc = (RtlCaptureContext_*)GetProcAddress(hinstLib, "RtlCaptureContext");
 	STACKFRAME frame;
 	CONTEXT context;
-	int limit = 50;
+	int limit = 70;
 	DWORD machineType;
 
 	memset(&frame, 0, sizeof(STACKFRAME));
@@ -267,7 +291,7 @@ std::string DumpBackTrace(void *processId)
  * @param exceptionRecord
  * @param contextRecord
  */
-void HandleException(struct _EXCEPTION_RECORD *exceptionRecord, struct _CONTEXT *contextRecord)
+void HandleException(const char *message,struct _EXCEPTION_RECORD *exceptionRecord, struct _CONTEXT *contextRecord)
 {
     std::string s;
 	fflush(stderr);
@@ -293,12 +317,21 @@ void HandleException(struct _EXCEPTION_RECORD *exceptionRecord, struct _CONTEXT
 	{
 		s+=DumpExceptionInfo(process, exceptionRecord, contextRecord);
 	}
+        fflush(stderr);
+	fflush(stdout);	
 
 	s+=DumpBackTrace(process);
+        fflush(stderr);
+	fflush(stdout);	
+        
+        const char *title;
+        if(!message) title="Crash";
+            else     title=message;
         if (myFatalFunction)
-		myFatalFunction("Crash", s.c_str());
+		myFatalFunction(title, s.c_str());
 	SymCleanup(process);
-
+        fflush(stderr);
+	fflush(stdout);	
 	exit(1);
 }
 /**
@@ -311,7 +344,7 @@ void HandleException(struct _EXCEPTION_RECORD *exceptionRecord, struct _CONTEXT
  */
 EXCEPTION_DISPOSITION ExceptionHandler(struct _EXCEPTION_RECORD *exceptionRecord, void *establisherFrame, struct _CONTEXT *contextRecord, void *dispatcherContext)
 {
-	HandleException(exceptionRecord, contextRecord);
+	HandleException("ExceptionHandler",exceptionRecord, contextRecord);
 }
 /**
  * 
@@ -320,8 +353,20 @@ EXCEPTION_DISPOSITION ExceptionHandler(struct _EXCEPTION_RECORD *exceptionRecord
  */
 LONG WINAPI ExceptionFilter(struct _EXCEPTION_POINTERS *exceptionInfo)
 {
-	HandleException(exceptionInfo->ExceptionRecord, exceptionInfo->ContextRecord);
+	HandleException("ExceptionFilter",exceptionInfo->ExceptionRecord, exceptionInfo->ContextRecord);
+        return EXCEPTION_CONTINUE_SEARCH;
 }
+/**
+ * 
+ * @param pExceptionInfo
+ * @return 
+ */
+LONG WINAPI TopLevelExceptionHandler(struct _EXCEPTION_POINTERS *exceptionInfo)
+{
+    HandleException("TopLevelExceptionHandler",exceptionInfo->ExceptionRecord, exceptionInfo->ContextRecord);
+    return EXCEPTION_CONTINUE_SEARCH;
+}
+
 /**
  * 
  * @param info
@@ -330,5 +375,7 @@ LONG WINAPI ExceptionFilter(struct _EXCEPTION_POINTERS *exceptionInfo)
  */
 void ADM_backTrack(const char *info, int lineno, const char *file)
 {	
-	HandleException(NULL, NULL);
+    char title[2048]={0};
+        snprintf(title,2000,"%s at line %d, file %s",info,lineno,file);
+	HandleException(title,NULL, NULL);
 }
