@@ -18,6 +18,12 @@
 #include "ADM_bitstream.h"
 #include "DIA_coreToolkit.h"
 
+
+#define ADM_CLEAR_MM7()         __asm__ volatile( "pxor %%mm7,%%mm7"      ::    )
+#define ADM_EMMS()              __asm__ volatile( "emms\n"                ::    )
+
+//#undef ADM_CPU_X86
+
 static uint8_t tinyAverage(uint8_t *dst, uint8_t *src1, uint8_t *src2,uint32_t l)
 {
 
@@ -58,22 +64,10 @@ int a1,a2;
         ww=l>>2;
         rr=l&3;
 
-
-#ifdef GCC_2_95_X
-         __asm__(
-                         "pxor %mm7,%mm7"
-                ::
-                 );
-#else
-         __asm__(
-                         "pxor %%mm7,%%mm7"
-                ::
-                 );
-#endif
-
+        ADM_CLEAR_MM7();
           for(int y=0;y<ww;y++)
           {
-                __asm__(
+                __asm__ volatile(
                         "movd           (%0),%%mm0 \n"
                         "movd           (%1),%%mm1 \n"
                         "punpcklbw      %%mm7,%%mm0 \n"
@@ -84,15 +78,13 @@ int a1,a2;
                         "movd           %%mm0,(%2) \n"
 
                 : : "r" (s1),"r" (s2),"r"(d1)
+                :"memory"
                 );
                         s1+=4;
                         s2+=4;
                         d1+=4;
                 }
-        __asm__(
-                        "emms\n"
-                ::
-                );
+       ADM_EMMS();
         if(rr) tinyAverage(d1, s1, s2,rr);
         return 1;
 }
@@ -158,10 +150,7 @@ uint8_t *s1,*s2,*d1;
         rr=width&3;
 
 
-         __asm__(
-                         "pxor %%mm7,%%mm7"
-                ::
-                 );
+         ADM_CLEAR_MM7();
 
           for(int y=0;y<height;y++)
           {
@@ -175,7 +164,7 @@ uint8_t *s1,*s2,*d1;
 
                 for(int x=0;x<ww;x++)
                 {
-                    __asm__(
+                    __asm__ volatile(
                             "movd           (%0),%%mm0 \n"
                             "movd           (%1),%%mm1 \n"
                             "punpcklbw      %%mm7,%%mm0 \n"
@@ -186,6 +175,7 @@ uint8_t *s1,*s2,*d1;
                             "movd           %%mm0,(%2) \n"
 
                     : : "r" (s1),"r" (s2),"r"(d1)
+                    :"memory"
                     );
                     s1+=4;
                     s2+=4;
@@ -195,11 +185,7 @@ uint8_t *s1,*s2,*d1;
                 src2+=stride2;
                 target+=stride;
            }
-        __asm__(
-                        "emms\n"
-                ::
-                );
-
+        ADM_EMMS();
         return true;
 }
 /**
@@ -229,7 +215,8 @@ uint8_t *s1,*s2,*d1;
                 }
 
                     
-                __asm__(
+                __asm__ volatile(
+                        
                         "1: \n"
                         "movq           (%0),%%mm0  \n"
                         "movq           (%1),%%mm1  \n"
@@ -239,19 +226,17 @@ uint8_t *s1,*s2,*d1;
                         "add           $8,%1      \n"
                         "add           $8,%2      \n"
                         "sub           $1,%3      \n"
-                        "jnz             1b        \n"
+                        "jnz           1b        \n"
+                        
 
                 : : "r" (s1),"r" (s2),"r"(d1),"r"(count)
+                :"memory"
                 );
                 src1+=stride1;
                 src2+=stride2;
                 target+=stride;
            }
-        __asm__(
-                        "emms\n"
-                ::
-                );
-
+        ADM_EMMS();
         return true;
 }
 
@@ -319,25 +304,16 @@ uint32_t result=0;
         ll=l>>2;
         rr=l&3;
 
-#ifdef GCC_2_95_X
-         __asm__(
-                         "pxor %mm7,%mm7\n"
-                         "pxor %mm3,%mm3\n"
-                         "movq "Mangle(noise64)", %mm6\n"
-                :: 
-                 );
-#else
-         __asm__(
+         __asm__ volatile(
                          "pxor %%mm7,%%mm7\n"
                          "pxor %%mm3,%%mm3\n"
                          "movq "Mangle(noise64)", %%mm6\n"
-                :: 
+                :::  "memory"
                  );
-#endif
 
           for(int y=0;y<ll;y++)
           {
-                __asm__(
+                __asm__ volatile(
                         "movd           (%0),  %%mm0 \n"
                         "movd           (%1),  %%mm1 \n"
                         "punpcklbw      %%mm7, %%mm0 \n"
@@ -371,6 +347,7 @@ uint32_t result=0;
                         "paddw          %%mm0, %%mm3 \n" /* PADDQ is SSE2 */
 
                 : : "r" (s1),"r" (s2)
+                :"memory"
                 );
                         s1+=4;
                         s2+=4;
@@ -378,7 +355,7 @@ uint32_t result=0;
          }
         // Pack result
 #if 1        
-                __asm__(
+                __asm__ volatile(
                        
                         "movd           %%mm3,(%0)\n"
                         "emms\n"
@@ -448,20 +425,10 @@ int ll,rr;
         
         d1=dst;
       
-#ifdef GCC_2_95_X
-        __asm__(
-                         "pxor %mm7,%mm7"
-                ::
-                 );
-#else
-        __asm__(
-                         "pxor %%mm7,%%mm7"
-                ::
-                 );
-#endif
+        ADM_CLEAR_MM7();
         for(int x=0;x<ll;x++)
-                {
-                        __asm__(
+        {
+                        __asm__ volatile(
                         "movd           (%0),%%mm0 \n"
                         "movd           (%1),%%mm1 \n"
                        
@@ -476,15 +443,13 @@ int ll,rr;
                         "packuswb       %%mm0,  %%mm0\n"
                         "movd           %%mm0,(%2) \n"                       
                         :: "r"(s1),"r"(s2),"r"(d1)
+                        :"memory"
                         );
                         s1+=4;
                         s2+=4;
                         d1+=4;
-                }
-                 __asm__(                       
-                        "emms\n"
-                :: 
-                );
+        }
+        ADM_EMMS();
         if(rr) tinySubstractMMX(d1, s1, s2,rr);
         return 1;
 }
@@ -594,8 +559,8 @@ static inline void yuv444_MMX(uint8_t *src,uint8_t *dst,int w,int h,int s)
 {
 static uint64_t __attribute__((used)) FUNNY_MANGLE(mask) = 0x00ff000000ff0000LL;
 
-    __asm__(" movq "Mangle(mask)", %%mm7\n" ::);
-    __asm__(" pxor %%mm6,%%mm6\n" ::);
+    __asm__ volatile(" movq "Mangle(mask)", %%mm7\n" ::);
+    __asm__ volatile(" pxor %%mm6,%%mm6\n" ::);
     
     int step=w/8;
     int left=w-8*step;
@@ -608,7 +573,7 @@ static uint64_t __attribute__((used)) FUNNY_MANGLE(mask) = 0x00ff000000ff0000LL;
         xdst=dst;
         for(int x=0;x<step;x++)
         {
-                        __asm__(
+                        __asm__ volatile(
                         "movq           (%0),%%mm0 \n"
                         "pand           %%mm7,%%mm0\n"
                         "movq           8(%0),%%mm1 \n"
@@ -629,6 +594,7 @@ static uint64_t __attribute__((used)) FUNNY_MANGLE(mask) = 0x00ff000000ff0000LL;
                         
                         
                         :: "r"(xsrc),"r"(xdst)
+                        :"memory"
                         );
                         xsrc+=32;
                         xdst+=8;
@@ -638,7 +604,7 @@ static uint64_t __attribute__((used)) FUNNY_MANGLE(mask) = 0x00ff000000ff0000LL;
         dst+=s;
         src+=4*w;
     }
-     __asm__( "emms\n"::  );
+     ADM_EMMS();
 
 }
 #endif
@@ -659,7 +625,7 @@ static inline void YUV444_chroma_MMX(uint8_t *src,uint8_t *dst,uint8_t *dst2,int
         xdst2=dst2;
         for(int x=0;x<step;x++)
         {
-                        __asm__(
+                        __asm__ volatile(
                         "movq           (%0),%%mm0 \n"
                         "movq           8(%0),%%mm1 \n"
                         "movq           16(%0),%%mm2 \n"
@@ -688,6 +654,7 @@ static inline void YUV444_chroma_MMX(uint8_t *src,uint8_t *dst,uint8_t *dst2,int
 
                         "movd           %%mm4,(%2) \n"                       
                         :: "r"(xsrc),"r"(xdst),"r"(xdst2)
+                        :"memory"
                         );
                         xsrc+=32;
                         xdst+=4;
@@ -702,7 +669,7 @@ static inline void YUV444_chroma_MMX(uint8_t *src,uint8_t *dst,uint8_t *dst2,int
         dst2+=s2;
         src+=4*w*4;
     }
-     __asm__( "emms\n"::  );
+     ADM_EMMS();
 
 }
 #endif
@@ -723,7 +690,7 @@ static inline void YUV444_chroma_C(uint8_t *src,uint8_t *dst,int w,int h,int s)
 
 #ifdef ADM_CPU_X86
 /**
- * \fn nv12_to_uv_mmx
+ * \fn uv_to_nv12_mmx
  * \brief unpack nv12 interleaved uv into planar uv
  * @param w
  * @param h
@@ -734,7 +701,7 @@ static inline void YUV444_chroma_C(uint8_t *src,uint8_t *dst,int w,int h,int s)
  * @param srcPitch
  * @param src
  */
-static void uv_to_nv12_mmx(int w, int h,int upitch, int vpitch, uint8_t *srcu, uint8_t *srcv,int strideUV, uint8_t *dst)
+static void uv_to_nv12_mmx(int w, int h,int upitch, int vpitch, uint8_t *srcv, uint8_t *srcu,int strideUV, uint8_t *dst)
 {
         int mod8=w>>3;
         int leftOver=w&7;
@@ -747,25 +714,28 @@ static void uv_to_nv12_mmx(int w, int h,int upitch, int vpitch, uint8_t *srcu, u
                 dst+=strideUV;
                 srcu+=upitch;
                 srcv+=vpitch;   
-                x=mod8;
-                      __asm__(
-                        "1:"
+                      __asm__ volatile(
+                        "mov            %4,%3      \n" // local copy
+                        "1:\n"
                         "movq           (%1),%%mm0   \n" // U
                         "movq           (%2),%%mm1   \n" // V
                         "movq           %%mm0,%%mm2  \n"                       
                         "movq           %%mm1,%%mm3  \n"   
 
-                        "punpcklbw      %%mm1,%%mm0 \n"
-                        "punpckhbw      %%mm3,%%mm2 \n"
-                        "movq           %%mm0,(%0)  \n"                       
+                        "punpcklbw      %%mm1,%%mm0  \n"
+                        "punpckhbw      %%mm3,%%mm2  \n"
+                        "movq           %%mm0,(%0)   \n"                       
                         "movq           %%mm2,8(%0)  \n"     
                         
-                        "add            $16,%0\n"
-                        "add            $8,%1\n"
-                        "add            $8,%2\n"
-                        "sub            $1,%3\n"
-                        "jnz            1b\n"
-                        :: "r"(ddst),"r"(u),"r"(v),"r"(x)
+                        "add            $16,%0       \n"
+                        "add            $8,%1        \n"
+                        "add            $8,%2        \n"
+                        "sub            $1,%3        \n"
+                        "jnz            1b           \n"
+                        
+                        :
+                        : "r"(ddst),"r"(u),"r"(v),"r"(x),"r"(mod8)
+                        : "memory"
                         );
                 if(leftOver)
                 {
@@ -781,17 +751,65 @@ static void uv_to_nv12_mmx(int w, int h,int upitch, int vpitch, uint8_t *srcu, u
                     }
                 }
         }
-        __asm__(
-                "emms\n"
-                ::
-            );
+        ADM_EMMS();
         return ;
 }
 
+/**
+ * \fn nv12_to_uv_mmx
+ * @param w
+ * @param h
+ * @param upitch
+ * @param vpitch
+ * @param dstu
+ * @param dstv
+ * @param srcPitch
+ * @param src
+ */
+
+static void nv12_to_u_v_mmx_one_line(int w8, uint8_t *dstu, uint8_t *dstv, uint8_t *src)
+{
+      __asm__ volatile(
+
+
+                  "1:\n"                  
+                  "movq           (%4),%%mm0   \n"
+                  "movq           8(%4),%%mm1  \n"                              
+                  "movq           %%mm0,%%mm2  \n"                       
+                  "movq           %%mm1,%%mm3  \n"   
+
+                  "psllw          $8,%%mm0    \n"                              
+                  "psrlw          $8,%%mm0    \n"    
+
+                  "psllw          $8,%%mm1    \n"                              
+                  "psrlw          $8,%%mm1    \n"    
+
+
+                  "packuswb       %%mm1,%%mm0 \n"
+
+                  "psrlw          $8,%%mm2    \n"                              
+                  "psrlw          $8,%%mm3    \n"    
+
+                  "packuswb       %%mm3,%%mm2 \n"
+
+                  "movq           %%mm0,(%6)  \n"                       
+                  "movq           %%mm2,(%5)  \n"     
+
+                  "add            $16,%4\n"
+                  "add            $8,%5\n"
+                  "add            $8,%6\n"
+                  "sub            $1,%7\n"
+                  "jnz            1b\n"
+
+                  : "=r"(src),"=r"(dstu),"=r"(dstv),"=r"(w8) //00..3
+                  : "0"(src),"1"(dstu),"2"(dstv),"3"(w8)     //4..7
+                  : "memory"
+                  );
+}
 
 static void nv12_to_uv_mmx(int w, int h,int upitch, int vpitch, uint8_t *dstu, uint8_t *dstv,int srcPitch, uint8_t *src)
 {
-        int mod16=w>>3;
+        int mod8=w>>3;
         int leftOver=w&7;
         int x;
         for(int y=0;y<h;y++)
@@ -803,42 +821,14 @@ static void nv12_to_uv_mmx(int w, int h,int upitch, int vpitch, uint8_t *dstu, u
                 dstu+=upitch;
                 dstv+=vpitch;                        
 
-                        x=mod16;
-                      __asm__(
-                        "1:"
-                        "movq           (%0),%%mm0   \n"
-                        "movq           8(%0),%%mm1  \n"                              
-                        "movq           %%mm0,%%mm2  \n"                       
-                        "movq           %%mm1,%%mm3  \n"   
-
-                        "psllw          $8,%%mm0    \n"                              
-                        "psrlw          $8,%%mm0    \n"    
-
-                        "psllw          $8,%%mm1    \n"                              
-                        "psrlw          $8,%%mm1    \n"    
-
-                              
-                        "packuswb       %%mm1,%%mm0 \n"
-                        
-                        "psrlw          $8,%%mm2    \n"                              
-                        "psrlw          $8,%%mm3    \n"    
-                              
-                        "packuswb       %%mm3,%%mm2 \n"
-                        
-                        "movq           %%mm0,(%2)  \n"                       
-                        "movq           %%mm2,(%1)  \n"     
-                        
-                        "add            $16,%0\n"
-                        "add            $8,%1\n"
-                        "add            $8,%2\n"
-                        "sub            $1,%3\n"
-                        "jnz            1b\n"
-                        :: "r"(ssrc),"r"(u),"r"(v),"r"(x)
-                        );
+                nv12_to_u_v_mmx_one_line(mod8,u,v,ssrc);
                 if(leftOver)
                 {
-                    x=mod16*8;
-                    for(;x<w;x++)
+                    int c=mod8*8;
+                    ssrc+=mod8*16;
+                    u+=mod8*8;
+                    v+=mod8*8;
+                    for(;c<w;c++)
                     {
                         *u++=ssrc[1];
                         *v++=ssrc[0];
@@ -846,11 +836,7 @@ static void nv12_to_uv_mmx(int w, int h,int upitch, int vpitch, uint8_t *dstu, u
                     }
                 }
         }
-        __asm__(
-                "emms\n"
-                ::
-            );
-
+        ADM_EMMS();
 }
 
 #endif
@@ -905,7 +891,7 @@ bool    ADMImage::convertFromNV12(uint8_t *yData, uint8_t *uvData, int strideY,
         h/=2;
         w/=2;
         
-        #ifdef ADM_CPU_X86
+        #if defined(ADM_CPU_X86) && 1
                 if(CpuCaps::hasMMX())
                     nv12_to_uv_mmx(w,h,GetPitch(PLANAR_U),GetPitch(PLANAR_V),GetWritePtr(PLANAR_U),GetWritePtr(PLANAR_V),strideUV,uvData);
                 else
@@ -958,8 +944,8 @@ static void uv_to_nv12_c(int w, int h,int upitch, int vpitch, uint8_t *srcu, uin
 
                 for(int x=0;x<w;x++)
                 {
-                    d[0]=*u++;
-                    d[1]=*v++;
+                    d[0]=*v++;
+                    d[1]=*u++;
                     d+=2;
                 }
         }
@@ -974,7 +960,7 @@ bool    ADMImage::interleaveUV(uint8_t *target, int stride)
 {    
         int w=_width/2;
         int h=_height/2;    
-#if defined(ADM_CPU_X86) && 0
+#if defined(ADM_CPU_X86) && 1
         if(CpuCaps::hasMMX())
             uv_to_nv12_mmx(w,h,GetPitch(PLANAR_U),GetPitch(PLANAR_V),GetWritePtr(PLANAR_U),GetWritePtr(PLANAR_V),stride,target);
         else
@@ -995,7 +981,7 @@ bool ADMImage::convertFromYUV444(uint8_t *from)
     uint8_t *dst=this->GetWritePtr(PLANAR_Y);
     uint8_t *src=from;
 
-    #ifdef ADM_CPU_X86
+    #if defined(ADM_CPU_X86) && 1
         if(CpuCaps::hasMMX())
             yuv444_MMX(src,dst,width,height,stride);            
         else
