@@ -104,7 +104,7 @@ ADMImage *createImageFromFile_jpeg(const char *filename)
 
 		    fseek(fd, 0, SEEK_SET);
 		    read16(fd);	// skip jpeg ffd8
-		    while (count < 10 && tag != 0xFFC0)
+		    while (count < 15 && tag != 0xFFC0)
 		    {
 
 		    	tag = read16(fd);
@@ -392,8 +392,8 @@ ADM_PICTURE_TYPE ADM_identifyImageFile(const char *filename,uint32_t *w,uint32_t
 		    			    		read8(fd);	// precision
 		    			    		*h = read16(fd);
 		    			    		*w = read16(fd);
-		    		                if(*w&1) *w++;
-		    		                if(*h&1) *h++;
+		    		                if(*w&1) w++;
+		    		                if(*h&1) h++;
 		    			    	}
 		    			    	else
 		    			    	{
