@@ -13,6 +13,9 @@ FEATURES_VDPAU, 	//bool
 FEATURES_XVBA, 	//bool
 FEATURES_LIBVA, 	//bool
 FEATURES_ENABLE_OPENGL, 	//bool
+FEATURES_CAP_REFRESH_ENABLED, 	//bool
+FEATURES_CAP_REFRESH_VALUE, 	//uint32_t
+FEATURES_SDLDRIVER, 	//string
 LASTFILES_LASTDIR_READ, 	//string
 LASTFILES_LASTDIR_WRITE, 	//string
 LASTFILES_FILE1, 	//string
@@ -36,6 +39,7 @@ DEFAULT_POSTPROC_TYPE, 	//uint32_t
 DEFAULT_POSTPROC_VALUE, 	//uint32_t
 DEFAULT_DOWNMIXING, 	//uint32_t
 DEFAULT_LANGUAGE, 	//string
+DEFAULT_WARN_FOR_FONTS, 	//bool
 MPEGSPLIT_AUTOSPLIT, 	//uint32_t
 AVISYNTH_AVISYNTH_ALWAYS_ASK, 	//bool
 AVISYNTH_AVISYNTH_DEFAULTPORT, 	//uint32_t
