@@ -459,7 +459,7 @@ VAImage   *admLibVA::allocateNV12Image( int w, int h)
     int xError=1;
     CHECK_WORKING(NULL);
     VAImage *image=new VAImage;
-    memset(image,0,sizeof(image));
+    memset(image,0,sizeof(*image));
     CHECK_ERROR(vaCreateImage ( ADM_coreLibVA::display, &ADM_coreLibVA::imageFormatNV12,
                 w,    h,    
                 image));
@@ -482,7 +482,7 @@ VAImage   *admLibVA::allocateYV12Image( int w, int h)
     int xError=1;
     CHECK_WORKING(NULL);
     VAImage *image=new VAImage;
-    memset(image,0,sizeof(image));
+    memset(image,0,sizeof(*image));
     CHECK_ERROR(vaCreateImage ( ADM_coreLibVA::display, &ADM_coreLibVA::imageFormatYV12,
                 w,    h,    
                 image));
@@ -545,7 +545,13 @@ VASurfaceID        admLibVA::allocateSurface(int w, int h)
        
        aprintf("Creating surface %d x %d\n",w,h);
        VASurfaceID s;
-        CHECK_ERROR(vaCreateSurfaces(ADM_coreLibVA::display,w,h,VA_RT_FORMAT_YUV420,1,&s));
+ 
+        CHECK_ERROR(vaCreateSurfaces(ADM_coreLibVA::display,
+                        VA_RT_FORMAT_YUV420,
+                        w,h,
+                        &s,1,
+                        NULL,0));
+
         if(!xError)
         {
             return s;
@@ -791,6 +797,7 @@ bool   admLibVA::downloadFromImage( ADMImage *src,VAImage *dest)
         case VA_FOURCC_YV12:
                 {
                         ADMImageRef ref(src->_width,src->_height);
+                        ref.copyInfo(src);
                         for(int i=0;i<3;i++)
                         {
                                 ref._planes[i]= ptr+dest->offsets[i];
@@ -799,22 +806,11 @@ bool   admLibVA::downloadFromImage( ADMImage *src,VAImage *dest)
                         src->duplicate(&ref);
                 }
                 break;
-        case VA_FOURCC_NV12:src->convertFromNV12(  ptr+dest->offsets[0], ptr+dest->offsets[1],dest->pitches[0],dest->pitches[1]);break;
+        case VA_FOURCC_NV12:
+                        src->convertFromNV12(  ptr+dest->offsets[0], ptr+dest->offsets[1],dest->pitches[0],dest->pitches[1]);
+                        break;
         default: ADM_assert(0);
     }
-int w=src->_width;
-    int h=src->_height;
-    int dstStride= dest->pitches[0];
-    int srcStride= src->GetPitch(PLANAR_Y);
-    uint8_t *s=    src->GetReadPtr(PLANAR_Y);
-    uint8_t *d=    ptr+dest->offsets[0];
-    for(int y=0;y<h;y++)
-    {
-        memcpy(s,d,w);
-        s+=srcStride;
-        d+=dstStride;
-    }
-    
     CHECK_ERROR(vaUnmapBuffer (ADM_coreLibVA::display,dest->buf));    
     return true;
 }
