@@ -86,10 +86,11 @@ public:
     static VdpStatus mixerCreate(uint32_t width,uint32_t height, VdpVideoMixer *mixer,bool deinterlace=false);
     static VdpStatus mixerDestroy(VdpVideoMixer mixer);
     static VdpStatus mixerRender(VdpVideoMixer mixer,VdpVideoSurface sourceSurface,VdpOutputSurface targetOutputSurface, uint32_t targetWidth, uint32_t targetHeight );
+    static VdpStatus mixerRenderWithCropping(VdpVideoMixer mixer,VdpVideoSurface sourceSurface,VdpOutputSurface targetOutputSurface, uint32_t targetWidth, uint32_t targetHeight ,
+                                            uint32_t sourceWidth, uint32_t sourceHeight);
     static VdpStatus mixerRenderWithPastAndFuture(bool topField,VdpVideoMixer mixer,
                                 VdpVideoSurface sourceSurface[3], // Past present future
-                                VdpOutputSurface targetOutputSurface,
-                                uint32_t targetWidth, uint32_t targetHeight );
+                                VdpOutputSurface targetOutputSurface, uint32_t targetWidth,     uint32_t targetHeight,uint32_t sourceWidth,     uint32_t sourceHeight );
     static VdpStatus mixerGetAttributesValue(VdpVideoMixer mixer,
                                 uint32_t attrCount,
                                 const  VdpVideoMixerAttribute *xkeys,
@@ -102,6 +103,9 @@ public:
     {
                 return (x+15) & ~15;
     }
+    static VdpStatus outputSurfaceGetParameters(  VdpOutputSurface surface,    VdpRGBAFormat *  rgba_format,
+                            uint32_t *       width,    uint32_t *       height);
+    static VdpStatus surfaceGetParameters(VdpVideoSurface surface,VdpChromaType *chomra,uint32_t *w,uint32_t *h);
 
 #endif
 };
