@@ -440,6 +440,9 @@ bool   decoderFF::uncompress (ADMCompressedImage * in, ADMImage * out)
     case PIX_FMT_YUVJ422P:
       out->_colorspace = ADM_COLOR_YUV422P;
       break;
+    case PIX_FMT_GRAY8:
+       out->_colorspace = ADM_COLOR_Y8;
+       break;
     case PIX_FMT_YUV444P:
     case PIX_FMT_YUVJ444P:
       out->_colorspace = ADM_COLOR_YUV444;
@@ -613,6 +616,16 @@ extern "C"
 {
   void adm_lavLogCallback(void  *instance, int level, const char* fmt, va_list list);
 }
+
+
+extern "C" 
+{
+static  void ffFatalError(const char *what,int lineno, const char *filez)
+{
+        ADM_backTrack(what,lineno,filez);
+}
+
+}
 /**
     \fn ADM_lavInit
     \brief Init both lavcodec and lavformat
@@ -621,7 +634,7 @@ void ADM_lavInit(void)
 {
     avcodec_register_all();
     av_log_set_callback(adm_lavLogCallback);
-
+    av_setFatalHandler(ffFatalError);
 #ifdef ADM_DEBUG
     av_log_set_level(AV_LOG_DEBUG);
 #endif
