@@ -17,7 +17,9 @@
 #include "ADM_default.h"
 #include "ADM_Video.h"
 #include "ADM_mkv.h"
+#include "ADM_codecType.h"
 #include "mkv_tags.h"
+
 /**
     \fn open
     \brief Try to open the mkv file given as parameter
@@ -61,14 +63,23 @@ uint8_t mkvHeader::open(const char *name)
             _timeBase=timeBase/1000; // We work in us
         }
   }
-  /* Now find tracks */
-  if(!ebml.find(ADM_MKV_SECONDARY,MKV_SEGMENT,MKV_TRACKS,&alen))
+  /* --*/
+  if(!ebml.simplefind(MKV_SEGMENT,&len,true))
   {
-     printf("[MKV] Cannot find tracks\n");
-    return 0;
+      printf("[MKV] Cannot find Segment\n");      
+      return false;
   }
+  _segmentPosition=ebml.tell();
+  printf("[MKV] found Segment at 0x%llx\n",(uint64_t)_segmentPosition);
+   /* Now find tracks */
+  if(ebml.find(ADM_MKV_SECONDARY,MKV_SEGMENT,MKV_SEEK_HEAD,&alen))
+  {
+       ADM_ebml_file seekHead( &ebml,alen);
+       readSeekHead(&seekHead);       
+  }
+  /* Now find tracks */
   /* And analyze them */
-  if(!analyzeTracks(&ebml,alen))
+  if(!analyzeTracks(&ebml))
   {
       printf("[MKV] incorrect tracks\n");
   }
@@ -78,12 +89,13 @@ uint8_t mkvHeader::open(const char *name)
     printf("[MKV] No video\n");
     return 0;
   }
+  readCue(&ebml);
   printf("[MKV] Indexing clusters\n");
   if(!indexClusters(&ebml))
   {
     printf("[MKV] Cluster indexing failed\n");
     return 0;
-  }
+  }  
   printf("[MKV]Found %u clusters\n",_clusters.size());
   printf("[MKV] Indexing video\n");
     if(!videoIndexer(&ebml))
@@ -94,6 +106,13 @@ uint8_t mkvHeader::open(const char *name)
   // update some infos
   _videostream.dwLength= _mainaviheader.dwTotalFrames=_tracks[0].index.size();;
 
+    if(!isH264Compatible(_videostream.fccHandler) && !isMpeg4Compatible(_videostream.fccHandler) && !isMpeg12Compatible(_videostream.fccHandler))
+    {
+        updateFlagsWithCue();
+    }
+    _cueTime.clear();
+  
+  
   _parser=new ADM_ebml_file();
   ADM_assert(_parser->open(name));
   _filename=ADM_strdup(name);
@@ -165,7 +184,10 @@ uint8_t mkvHeader::open(const char *name)
           duration/=1000;
           uint32_t duration32=(uint32_t)duration;
           printf("[MKV] Video Track duration for %u ms\n",duration32);
-          // Useless.....readCue(&ebml);
+          // Useless.....
+
+        
+
           for(int i=0;i<_nbAudioTrack;i++)
           {
             rescaleTrack(&(_tracks[1+i]),duration32);
@@ -184,9 +206,8 @@ uint8_t mkvHeader::open(const char *name)
         _audioStreams[i]=ADM_audioCreateStream(&(_tracks[1+i].wavHeader), _access[i]);;
         _audioStreams[i]->setLanguage(_tracks[1+i].language);
     }
-  
+  //dumpVideoIndex(200);
   printf("[MKV]Matroska successfully read\n");
-
   return 1;
 }
 /**
@@ -331,18 +352,62 @@ uint8_t mkvHeader::checkHeader(void *head,uint32_t headlen)
  return 1;
 
 }
+
+/**
+ * \fn goBeforeAtomAtPosition
+ * \brief check and position the read at the payload for atom searchedId, return payloadSize in outputLen
+ */
+bool mkvHeader::goBeforeAtomAtPosition(ADM_ebml_file *parser, uint64_t position,uint64_t &outputLen, MKV_ELEM_ID searchedId,const char *txt)
+{
+    uint64_t id,len;
+    ADM_MKV_TYPE type;
+    const char *ss;
+
+    if(!position)
+    {
+        ADM_warning("No offset available for %s\n",txt);
+        return false;
+    }
+    parser->seek(position);
+    if(!parser->readElemId(&id,&len))
+    {
+        ADM_warning("No element  available for %s\n",txt);
+        return false;
+    }
+    if(!ADM_searchMkvTag( (MKV_ELEM_ID)id,&ss,&type))
+    {
+      printf("[MKV/SeekHead] Tag 0x%"PRIx64" not found (len %"PRIu64")\n",id,len);
+      return false;
+    }
+    if(id!=searchedId)
+    {
+        printf("Found %s instead of %s, ignored \n",ss,txt);
+        return false;
+    }    
+    outputLen=len;
+    return true;
+}
+
 /**
     \fn analyzeTracks
     \brief Read Tracks Info.
 */
-uint8_t mkvHeader::analyzeTracks(void *head,uint32_t headlen)
+bool mkvHeader::analyzeTracks(ADM_ebml_file *parser)
 {
-  uint64_t id,len;
-  ADM_MKV_TYPE type;
+  uint64_t len;
+  uint64_t id;
   const char *ss;
- ADM_ebml_file father( (ADM_ebml_file *)head,headlen);
- while(!father.finished())
- {
+  ADM_MKV_TYPE type;
+  
+  if(!goBeforeAtomAtPosition(parser, _trackPosition,len, MKV_TRACKS,"MKV_TRACKS"))
+  {
+      ADM_warning("Cannot go to the TRACKS atom\n");
+      return false;
+  }
+  
+    ADM_ebml_file father( parser,len);
+    while(!father.finished())
+    {
       father.readElemId(&id,&len);
       if(!ADM_searchMkvTag( (MKV_ELEM_ID)id,&ss,&type))
       {
@@ -358,7 +423,7 @@ uint8_t mkvHeader::analyzeTracks(void *head,uint32_t headlen)
         continue;
       }
       if(!analyzeOneTrack(&father,len)) return 0;
- }
+    }
  return 1;
 }
 
@@ -531,6 +596,9 @@ uint8_t mkvHeader::close(void)
   _audioStreams=NULL;
   
   readBuffer=NULL;
+  _cuePosition=0;
+  _segmentPosition=0;
+  _trackPosition=0;
 }
 /**
     \fn ~mkvHeader
@@ -753,8 +821,76 @@ bool    mkvHeader::setPtsDts(uint32_t frame,uint64_t pts,uint64_t dts)
     return true;
 
 }
+/**
+ * \fn readSeekHead
+ * \bried used to locate the interesting parts of the file
+ */
+bool    mkvHeader::readSeekHead(ADM_ebml_file *body)
+{
+    uint64_t vlen,len;
+    ADM_info("Parsing SeekHead\n");
+    while(!body->finished())
+    {
+        if(!body->simplefind(MKV_SEEK,&vlen,false))
+             break;         
+        ADM_ebml_file item(body,vlen);              
+        uint64_t id;
+        ADM_MKV_TYPE type;
+        const char *ss;
 
-
+        item.readElemId(&id,&len);
+        if(!ADM_searchMkvTag( (MKV_ELEM_ID)id,&ss,&type))
+        {
+          printf("[MKV/SeekHead] Tag 0x%"PRIx64" not found (len %"PRIu64")\n",id,len);
+          return false;
+        }
+        if(id!=MKV_ID)
+        {
+          printf("Found %s in CUES, ignored \n",ss);
+          item.skip(len);
+          return false;
+        }
+        // read id
+        uint64_t t=item.readEBMCode_Full();
+        if(!ADM_searchMkvTag( (MKV_ELEM_ID)t,&ss,&type))
+        {
+          printf("[MKV/SeekHead] Tag 0x%"PRIx64" not found (len %"PRIu64")\n",id,len);
+          return false;
+        }
+        ADM_info("Found entry for %s\n",ss);
+        item.readElemId(&id,&len);
+        if(!ADM_searchMkvTag( (MKV_ELEM_ID)id,&ss,&type))
+        {
+          printf("[MKV/SeekHead] Tag 0x%"PRIx64" not found (len %"PRIu64")\n",id,len);
+          return false;
+        }
+        if(id!=MKV_SEEK_POSITION)
+        {
+          printf("Found %s in CUES, ignored \n",ss);
+          item.skip(len);
+          return false;
+        }
+        uint64_t position=item.readUnsignedInt(len);
+        switch(t)
+        {
+            case MKV_CUES:
+                    _cuePosition=position+_segmentPosition;
+                    ADM_info("   at position  0x%llx\n",_cuePosition);
+                    break;
+            case MKV_TRACKS:
+                    _trackPosition=position+_segmentPosition;;
+                    ADM_info("   at position at 0x%llx\n",_trackPosition);
+            case MKV_INFO:
+            default:
+                    break;
+        }
+              
+    }
+    ADM_info("Parsing SeekHead done successfully\n");
+    if(!_trackPosition )
+        return false;
+    return true;
+}
 
 //****************************************
 //EOF
