@@ -133,7 +133,7 @@ bool bAppend=false;
 
     
     pkt=new tsPacketLinearTracker(videoTrac->trackPid, audioTracks);
-
+    
     FP_TYPE append=FP_DONT_APPEND;
     if(true==ADM_probeSequencedFile(file))
     {
@@ -146,11 +146,13 @@ bool bAppend=false;
     pkt->open(file,append);
     data.pkt=pkt;
     fullSize=pkt->getSize();
+    gui=createProcessing("Indexing",pkt->getSize());
     int lastRefIdc=0;
+    bool keepRunning=true;
     //******************
     // 1 search SPS
     //******************
-      while(1)
+      while(keepRunning)
       {
         int startCode=pkt->findStartCode();
 
@@ -177,7 +179,8 @@ bool bAppend=false;
             pkt->seek(tmpInfo.startAt,tmpInfo.offset-5);
             if (extractSPSInfo(SEI_nal.payload, SEI_nal.payloadSize-4,&spsInfo))
             {
-              printf("[TsIndexer] Found video %"PRIu32"x%"PRIu32", fps=%"PRIu32"\n",video.w,video.h,video.fps);
+              ADM_info("[TsIndexer] Found video %"PRIu32"x%"PRIu32", fps=%"PRIu32"\n",video.w,video.h,video.fps);
+              ADM_info("[TsIndexer] SPS says %"PRIu32"x%"PRIu32"\n",spsInfo.width,spsInfo.height);
               seq_found=1;
               video.w=spsInfo.width;
               video.h=spsInfo.height;
@@ -201,7 +204,7 @@ bool bAppend=false;
     // 2 Index
     //******************
         bool fourBytes;
-      while(1)
+      while(keepRunning)
       {
           fourBytes=false;
         int startCode=pkt->findStartCode2(fourBytes);
@@ -262,7 +265,8 @@ resume:
                             decodingImage=false;
                             pkt->getInfo(&thisUnit.packetInfo);
                             thisUnit.consumedSoFar=pkt->getConsumed();
-                            addUnit(data,unitTypeSei,thisUnit,startCodeLength+SEI_nal.payloadSize+1);                            
+                            if(!addUnit(data,unitTypeSei,thisUnit,startCodeLength+SEI_nal.payloadSize+1))
+                                keepRunning=false;
                             fourBytes=true;
                             goto resume;
                             }
@@ -277,7 +281,8 @@ resume:
                                     firstSps=false;
                                 }
                                 thisUnit.consumedSoFar=pkt->getConsumed();
-                                addUnit(data,unitTypeSps,thisUnit,startCodeLength);
+                                if(!addUnit(data,unitTypeSps,thisUnit,startCodeLength))
+                                    keepRunning=false;
                           break;
 
                   case NAL_IDR:
@@ -326,7 +331,8 @@ resume:
                       pkt->getInfo(&thisUnit.packetInfo);
                       thisUnit.consumedSoFar=pkt->getConsumed();
 
-                      addUnit(data,unitTypePic,thisUnit,startCodeLength+NON_IDR_PRE_READ);
+                      if(!addUnit(data,unitTypePic,thisUnit,startCodeLength+NON_IDR_PRE_READ))
+                          keepRunning=false;
                         // reset to default
                       thisUnit.imageStructure=pictureFrame;
                       thisUnit.recoveryCount=0xff;
